VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "acls"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Dim g_sErrInfo As String
Dim g_bCancel As Boolean

Public Event Status(sStatusInfo As String, ByRef bCancel As Boolean)

Public Property Get ErrInfo() As String
    ErrInfo = g_sErrInfo
End Property

Public Function Modify(sFileOrFolder As String, sUserIdOrGroup As String, _
                       dwAccessMask As Long, dwAccessMode As Long, _
                       Optional bRecursive As Boolean) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sFileOrFolder = "" Or sUserIdOrGroup = "" Or _
        dwAccessMask = 0 Or dwAccessMode = 0) Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        Modify = 1
        Exit Function
    End If
    
    If (Len(sFileOrFolder) > MAX_PATH) Then
        g_sErrInfo = "The sFileOrFolder parameter was passed with length greater than MAX_PATH = " & MAX_PATH & "."
        Modify = 1
        Exit Function
    End If
                    
    Dim oFs As Object, oFolder As Object, oFile As Object, _
        sFolder As String, sWildCard As String, _
        iStart As Integer, iEnd As Integer, bDone As Boolean, _
        bFound As Boolean, bAllFiles As Boolean, bWildCardMatch As Boolean, _
        lRet As Long
        
    Dim sDllPath As String, hModule As Long
        
    sDllPath = App.Path & "\icuUtils.dll"
    hModule = LoadLibrary(sDllPath)
    If (hModule = 0) Then
        g_sErrInfo = "Error Loading Library " & Chr(34) & sDllPath & Chr(34) & "."
        Modify = 1
        Exit Function
    End If
    
    Set oFs = CreateObject("Scripting.FileSystemObject")
    bFound = False
    bAllFiles = False
    bWildCardMatch = False
       
    If (oFs.FileExists(sFileOrFolder)) Then '// user provided a single file name
        bFound = True
        '// use AclModify Api to apply setting
        lRet = AclModify(sFileOrFolder, sUserIdOrGroup, _
                         dwAccessMask, dwAccessMode, True)
        If (lRet) Then
            g_sErrInfo = sFileOrFolder & " Acl setting failed."
            Set oFs = Nothing
            FreeLibrary (hModule)
            Modify = 1
            Exit Function
        End If
                                
    ElseIf (oFs.FolderExists(sFileOrFolder)) Then '// user provided a folder name
        bFound = True
        '// use AclModify Api to apply setting
        lRet = AclModify(sFileOrFolder, sUserIdOrGroup, _
                         dwAccessMask, dwAccessMode, False)
        If (lRet) Then
            g_sErrInfo = sFileOrFolder & " Acl setting failed."
            Set oFs = Nothing
            FreeLibrary (hModule)
            Modify = 1
            Exit Function
        End If
        
        '// recurse setting on all files one level deep if bRecursive = True
        If (bRecursive) Then
            Set oFolder = oFs.GetFolder(sFileOrFolder)
            For Each oFile In oFolder.Files
                '// use AclModify Api to apply setting
                lRet = AclModify(oFile.Path, sUserIdOrGroup, _
                                 dwAccessMask, dwAccessMode, True)
                If (lRet) Then
                    g_sErrInfo = oFile.Path & " Acl setting failed."
                    Set oFile = Nothing
                    Set oFolder = Nothing
                    Set oFs = Nothing
                    FreeLibrary (hModule)
                    Modify = 1
                    Exit Function
                End If
            Next '// oFile
        End If
        
    Else '// user may have provided a wildcard specification or bad file[path] name
        iStart = 0
        bDone = False
        While Not (bDone)
            iEnd = 0
            iEnd = InStr(iStart + 1, sFileOrFolder, "\", vbBinaryCompare)
            If (iEnd = 0) Then
                bDone = True
                sFolder = Left(sFileOrFolder, iStart - 1)
                sWildCard = Mid(sFileOrFolder, iStart + 1)
                If (Not oFs.FolderExists(sFolder)) Then
                    g_sErrInfo = "ERROR_FILE[PATH]_NOT_FOUND or unsupported pre*.ext wildcard passed for sFileOrFolder = " & sFileOrFolder
                    Set oFile = Nothing
                    Set oFolder = Nothing
                    Set oFs = Nothing
                    FreeLibrary (hModule)
                    Modify = -1
                    Exit Function
                End If
                iStart = InStr(1, sWildCard, "*", vbBinaryCompare)
                If (iStart = 1) Then
                    iEnd = InStr(iStart + 1, sWildCard, ".*", vbBinaryCompare)
                    If (iEnd) Then
                        bAllFiles = True '// i.e. *.* passed
                    Else
                        iEnd = InStr(iStart + 1, sWildCard, ".", vbBinaryCompare)
                        sWildCard = Mid(sWildCard, iEnd) '// i.e. *.ext passed
                    End If
                Else
                    iStart = InStr(1, sWildCard, "*.*", vbBinaryCompare)
                    If (iStart) Then
                        sWildCard = Mid(sWildCard, 1, (iStart - 1)) '// i.e. pre*.* passed
                    Else
                        '// user likely passed pre*.ext which we currently do not support
                        g_sErrInfo = "ERROR_FILE[PATH]_NOT_FOUND or unsupported pre*.ext wildcard passed for sFileOrFolder = " & sFileOrFolder
                        Set oFile = Nothing
                        Set oFolder = Nothing
                        Set oFs = Nothing
                        FreeLibrary (hModule)
                        Modify = -1
                        Exit Function
                    End If
                End If
            Else
                iStart = iEnd
            End If
        Wend
        
        Set oFolder = oFs.GetFolder(sFolder)
        For Each oFile In oFolder.Files
            If (bAllFiles) Then
                '// use AclModify Api to apply setting
                lRet = AclModify(oFile.Path, sUserIdOrGroup, _
                                 dwAccessMask, dwAccessMode, True)
                If (lRet) Then
                    g_sErrInfo = oFile.Path & " Acl setting failed."
                    Set oFile = Nothing
                    Set oFolder = Nothing
                    Set oFs = Nothing
                    FreeLibrary (hModule)
                    Modify = 1
                    Exit Function
                End If
            Else
                iStart = InStr(1, oFile.Name, sWildCard)
                If (iStart) Then
                    bWildCardMatch = True
                    '// use AclModify Api to apply setting
                    lRet = AclModify(oFile.Path, sUserIdOrGroup, _
                                     dwAccessMask, dwAccessMode, True)
                    If (lRet) Then
                        g_sErrInfo = oFile.Path & " Acl setting failed."
                        Set oFile = Nothing
                        Set oFolder = Nothing
                        Set oFs = Nothing
                        FreeLibrary (hModule)
                        Modify = 1
                        Exit Function
                    End If
                End If
            End If
        Next '// oFile
    End If
    
    Set oFile = Nothing
    Set oFolder = Nothing
    Set oFs = Nothing
    FreeLibrary (hModule)
    If (Not bFound) And (Not bAllFiles) And (Not bWildCardMatch) Then
        g_sErrInfo = sFolder & "\" & sWildCard & "produced ERROR_FILE_NOT_FOUND."
        Modify = -1  '// return error_file[path]_not_found status
    Else
        Modify = 0
    End If
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Set oFile = Nothing
    Set oFolder = Nothing
    Set oFs = Nothing
    FreeLibrary (hModule)
    Modify = -1
End Function
