VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "resource"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Dim g_sErrInfo As String
Dim g_bCancel As Boolean

Public Event Status(sStatusInfo As String, ByRef bCancel As Boolean)

Public Property Get ErrInfo() As String
    ErrInfo = g_sErrInfo
End Property

Public Function CheckForLockedFile(sFileOrFolder As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sFileOrFolder = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        CheckForLockedFile = 1
        Exit Function
    End If
    
    If (Len(sFileOrFolder) > MAX_PATH) Then
        g_sErrInfo = "The sFileOrFolder parameter was passed with length greater than MAX_PATH = " & MAX_PATH & "."
        CheckForLockedFile = 1
        Exit Function
    End If

    Dim oFs As Object, oFolder As Object, oFile As Object, _
        sFolder As String, sWildCard As String, _
        iStart As Integer, iEnd As Integer, bDone As Boolean, _
        bFound As Boolean, bAllFiles As Boolean, bWildCardMatch As Boolean, _
        lRet As Long
        
    Dim tSecurityAttributes As stSecurityAttributes, hHandle As Long
    
    Set oFs = CreateObject("Scripting.FileSystemObject")
    bFound = False
    bAllFiles = False
    bWildCardMatch = False
    
    If (oFs.FileExists(sFileOrFolder)) Then '// user provided a single file name
        bFound = True
        '// use CreateFile to check for a lock
        tSecurityAttributes.nLength = CB_SECURITY_ATTRIBUTES
        tSecurityAttributes.lpSecurityDescriptor = 0&
        tSecurityAttributes.bInheritHandle = False
        hHandle = 0&
        hHandle = CreateFile(sFileOrFolder, GENERIC_READ, _
                             FILE_SHARE_NO, tSecurityAttributes, OPEN_EXISTING, _
                             FILE_ATTRIBUTE_NORMAL, 0&)
        If (hHandle = INVALID_HANDLE_VALUE) Then
            lRet = GetLastError()
            If (lRet) Then
                lRet = ERROR_FILE_NOT_FOUND
                g_sErrInfo = sFileOrFolder & " returned ERROR_FILE_NOT_FOUND."
                CheckForLockedFile = -1  '// return error_file[path]_not_found status
            Else
                g_sErrInfo = sFileOrFolder & " returned INVALID_HANDLE_VALUE most likely because a file is locked."
                CheckForLockedFile = 1  '// return filelocked status
            End If
            lRet = CloseHandle(hHandle)
            Set oFs = Nothing
            Exit Function
        Else
            lRet = CloseHandle(hHandle)
        End If
        
    ElseIf (oFs.FolderExists(sFileOrFolder)) Then '// user provided a folder name
        bFound = True
        Set oFolder = oFs.GetFolder(sFileOrFolder)
        For Each oFile In oFolder.Files
            '// use CreateFile to check for a lock
            tSecurityAttributes.nLength = CB_SECURITY_ATTRIBUTES
            tSecurityAttributes.lpSecurityDescriptor = 0&
            tSecurityAttributes.bInheritHandle = False
            hHandle = 0&
            hHandle = CreateFile(oFile.Path, GENERIC_READ, _
                                 FILE_SHARE_NO, tSecurityAttributes, OPEN_EXISTING, _
                                 FILE_ATTRIBUTE_NORMAL, 0&)
            If (hHandle = INVALID_HANDLE_VALUE) Then
                lRet = GetLastError()
                If (lRet) Then
                    lRet = ERROR_FILE_NOT_FOUND
                    g_sErrInfo = oFile.Path & " returned ERROR_FILE_NOT_FOUND."
                    CheckForLockedFile = -1  '// return error_file[path]_not_found status
                Else
                    g_sErrInfo = oFile.Path & " returned INVALID_HANDLE_VALUE most likely because a file is locked."
                    CheckForLockedFile = 1  '// return filelocked status
                End If
                lRet = CloseHandle(hHandle)
                Set oFile = Nothing
                Set oFolder = Nothing
                Set oFs = Nothing
                Exit Function
            Else
                lRet = CloseHandle(hHandle)
            End If
        Next '// oFile
        
    Else '// user may have provided a wildcard specification or bad file[path] name
        iStart = 0
        bDone = False
        While Not (bDone)
            iEnd = 0
            iEnd = InStr(iStart + 1, sFileOrFolder, "\", vbBinaryCompare)
            If (iEnd = 0) Then
                bDone = True
                sFolder = Left(sFileOrFolder, iStart - 1)
                sWildCard = Mid(sFileOrFolder, iStart + 1)
                If (Not oFs.FolderExists(sFolder)) Then
                    g_sErrInfo = "ERROR_FILE[PATH]_NOT_FOUND or unsupported pre*.ext wildcard passed for sFileOrFolder = " & sFileOrFolder
                    Set oFile = Nothing
                    Set oFolder = Nothing
                    Set oFs = Nothing
                    CheckForLockedFile = -1
                    Exit Function
                End If
                iStart = InStr(1, sWildCard, "*", vbBinaryCompare)
                If (iStart = 1) Then
                    iEnd = InStr(iStart + 1, sWildCard, ".*", vbBinaryCompare)
                    If (iEnd) Then
                        bAllFiles = True '// i.e. *.* passed
                    Else
                        iEnd = InStr(iStart + 1, sWildCard, ".", vbBinaryCompare)
                        sWildCard = Mid(sWildCard, iEnd) '// i.e. *.ext passed
                    End If
                Else
                    iStart = InStr(1, sWildCard, "*.*", vbBinaryCompare)
                    If (iStart) Then
                        sWildCard = Mid(sWildCard, 1, (iStart - 1)) '// i.e. pre*.* passed
                    Else
                        '// user likely passed pre*.ext which we currently do not support
                        g_sErrInfo = "ERROR_FILE[PATH]_NOT_FOUND or unsupported pre*.ext wildcard passed for sFileOrFolder = " & sFileOrFolder
                        Set oFile = Nothing
                        Set oFolder = Nothing
                        Set oFs = Nothing
                        CheckForLockedFile = -1
                        Exit Function
                    End If
                End If
            Else
                iStart = iEnd
            End If
        Wend
        
        Set oFolder = oFs.GetFolder(sFolder)
        For Each oFile In oFolder.Files
            If (bAllFiles) Then
                '// use CreateFile to check for a lock
                tSecurityAttributes.nLength = CB_SECURITY_ATTRIBUTES
                tSecurityAttributes.lpSecurityDescriptor = 0&
                tSecurityAttributes.bInheritHandle = False
                hHandle = 0&
                hHandle = CreateFile(oFile.Path, GENERIC_READ, _
                                     FILE_SHARE_NO, tSecurityAttributes, OPEN_EXISTING, _
                                     FILE_ATTRIBUTE_NORMAL, 0&)
                If (hHandle = INVALID_HANDLE_VALUE) Then
                    lRet = GetLastError()
                    If (lRet) Then
                        lRet = ERROR_FILE_NOT_FOUND
                        g_sErrInfo = oFile.Path & " returned ERROR_FILE_NOT_FOUND."
                        CheckForLockedFile = -1  '// return error_file[path]_not_found status
                    Else
                        g_sErrInfo = oFile.Path & " returned INVALID_HANDLE_VALUE most likely because a file is locked."
                        CheckForLockedFile = 1  '// return filelocked status
                    End If
                    lRet = CloseHandle(hHandle)
                    Set oFile = Nothing
                    Set oFolder = Nothing
                    Set oFs = Nothing
                    Exit Function
                Else
                    lRet = CloseHandle(hHandle)
                End If
            Else
                iStart = InStr(1, oFile.Name, sWildCard)
                If (iStart) Then
                    bWildCardMatch = True
                    '// use CreateFile to check for a lock
                    tSecurityAttributes.nLength = CB_SECURITY_ATTRIBUTES
                    tSecurityAttributes.lpSecurityDescriptor = 0&
                    tSecurityAttributes.bInheritHandle = False
                    hHandle = 0&
                    hHandle = CreateFile(oFile.Path, GENERIC_READ, _
                                         FILE_SHARE_NO, tSecurityAttributes, OPEN_EXISTING, _
                                         FILE_ATTRIBUTE_NORMAL, 0&)
                    If (hHandle = INVALID_HANDLE_VALUE) Then
                        lRet = GetLastError()
                        If (lRet) Then
                            lRet = ERROR_FILE_NOT_FOUND
                            g_sErrInfo = oFile.Path & " returned ERROR_FILE_NOT_FOUND."
                            CheckForLockedFile = -1  '// return error_file[path]_not_found status
                        Else
                            g_sErrInfo = oFile.Path & " returned INVALID_HANDLE_VALUE most likely because a file is locked."
                            CheckForLockedFile = 1  '// return filelocked status
                        End If
                        lRet = CloseHandle(hHandle)
                        Set oFile = Nothing
                        Set oFolder = Nothing
                        Set oFs = Nothing
                        Exit Function
                    Else
                        lRet = CloseHandle(hHandle)
                    End If
                End If
            End If
        Next '// oFile
    End If
    
    Set oFile = Nothing
    Set oFolder = Nothing
    Set oFs = Nothing
    If (Not bFound) And (Not bAllFiles) And (Not bWildCardMatch) Then
        g_sErrInfo = sFolder & "\" & sWildCard & "produced ERROR_FILE_NOT_FOUND."
        CheckForLockedFile = -1  '// return error_file[path]_not_found status
    Else
        CheckForLockedFile = 0
    End If
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    lRet = CloseHandle(hHandle)
    Set oFile = Nothing
    Set oFolder = Nothing
    Set oFs = Nothing
    CheckForLockedFile = -1
End Function

Function GetDomain(Optional sNameSpace As String, Optional sMachineName As String) As String
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sNameSpace = "") Then
        sNameSpace = NS_WNT
    End If
       
    Dim oMachine As IADs, lRet As Long
    
    If (sMachineName = "") Then
            sMachineName = Environ("COMPUTERNAME")
    End If
    
    Set oMachine = GetObject(sNameSpace & "://" & sMachineName)
    
    GetDomain = Mid(oMachine.Parent, Len(sNameSpace & "://") + 1)
        
    Set oMachine = Nothing
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Set oMachine = Nothing
    GetDomain = ""
End Function

Function GetMachineName() As String
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    GetMachineName = Environ("COMPUTERNAME")
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    GetMachineName = ""
End Function

Function GetOs(Optional sNameSpace As String, Optional sDomain As String, Optional sMachineName As String) As String
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sNameSpace = "") Then
        sNameSpace = NS_WNT
    End If
       
    Dim oMachine As IADs
    
    If (sMachineName = "") Then
        sMachineName = Environ("COMPUTERNAME")
    End If
    
    If Not (sDomain = "") Then
        Set oMachine = GetObject(sNameSpace & "://" & sDomain & "/" & sMachineName)
    Else
        Set oMachine = GetObject(sNameSpace & "://" & sMachineName)
    End If
    
    GetOs = oMachine.OperatingSystem

    Set oMachine = Nothing
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Set oMachine = Nothing
    GetOs = ""
End Function

Function GetOsVer(Optional sNameSpace As String, Optional sDomain As String, Optional sMachineName As String) As String
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sNameSpace = "") Then
        sNameSpace = NS_WNT
    End If
       
    Dim oMachine As IADs
    
    If (sMachineName = "") Then
        sMachineName = Environ("COMPUTERNAME")
    End If
    
    If Not (sDomain = "") Then
        Set oMachine = GetObject(sNameSpace & "://" & sDomain & "/" & sMachineName)
    Else
        Set oMachine = GetObject(sNameSpace & "://" & sMachineName)
    End If
        
    GetOsVer = oMachine.OperatingSystemVersion
    
    Set oMachine = Nothing
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Set oMachine = Nothing
    GetOsVer = ""
End Function

