/*
   version = 20Jan99
   status = in progress

   The icuHost COMponent and this accompanying script allow you to setup new hosting
   servers with large ranges of IPs, users, folders, acls, ftpRoots and WebRoots so that
   new customers can be easily assigned to already provisioned services with a minimal
   amount of effort, downtime and risk to the existing customers already using the 
   host for their production services.  It applies the large ranges of services by
   asking for a starting IpAddress and then the range to add rather than requiring the
   script writer to build the complicated loops and IpAddress generation processes that
   would be otherwise necessary to execute these steps using COMponent interfaces that
   simply execute each of these provisioning steps on a 1 by 1 basis.  Since it also has
   matching delete methods then it is easy to remove the ranges as well.  This component
   expects that you have WntSrv + Sp3 (or newer) + NtOptPack10 (w/ windows scripting host)
   installed on the host.  Included in the COMponents download is a setup.bat that outlines
   the steps necessary to setup and register this COMponent for use.  Once installed you
   can use the sample "hostProv.js" script to test out its functionality.
   
   If the model outlined above does not match your needs then you may also find
   this COMponent and its interfaces useful for either of the following purposes.
	a)  For use in batch provisioning [and tear down] of load testing servers
  	    where you want to simulate for testing purposes XX number of shared 
	    hosting customers configured and operating on a given host.

	b)  As a source code example demonstrating how to Enumerate, Add and Delete
  	    Keys in the Metabase using the faster IMSAdminBase pure COM interfaces
	    versus the ActiveDs dual COM interfaces.

	*** Notes ****
	1.	If you do not have at least one file (i.e. banner.txt) in the root of your
		<sHostingFolder>\defaultContent folder then the "FileCopy" and "CustomerCreate"
		routines will fail when they attempt to wildcard copy all of the contents
		found there.  The wildcard approach was taken to save you from having to 
		denote file by file what contents of the "defaultContent" folder should be
		set for each new customer hosting folder.  See the source code for the 
		icuHost.dll component for a sample of what "Customers\defaultContent" might
		possibly contain for your environment.

	2.  If your <sHostingPrefix> + host ip number is not > your account policy minimum
		pwd length setting then when users created by "UserCreate" or "CustomerCreate"
		methods will leave user entry with "user must change password at next logon"
		enabled.  It's best to avoid this scenario by ensuring pwd values are equal to
		or longer than the current host account policy pwd minimum length setting.
*/


// check arguments passed into script
// debugger;
var oArgs = WScript.Arguments;  // creation of script arguments object
if (1 > oArgs.Count())
	lr = errHandler();
var sTask = oArgs(0);
oArgs = null;
sTask = sTask.toLowerCase();
switch (sTask)
{
	case "debug":
	case "create":
	case "delete":
		break;
	default:
		errHandler();
		break;
}


// global defines
// debugger;
var TRUE = 1, FALSE = 0, S_OK = 0, E_FAIL = 0x80004005, hr = E_FAIL;
var GENERIC_READ = 0x80000000, GENERIC_WRITE = 0x40000000, GENERIC_EXECUTE = 0x20000000;
var GENERIC_ALL = 0x10000000, FILE_GENERIC_READ = 0x00120089, FILE_GENERIC_WRITE = 0x00120116;
var FILE_GENERIC_EXECUTE = 0x001200a0, FILE_ALL_ACCESS = 0x001f03ff, DELETE = 0x00010000;
var GRANT_ACCESS = 0x1, SET_ACCESS = 0x2, DENY_ACCESS = 0x3, REVOKE_ACCESS = 0x4;
var MD_ACCESS_READ = 0x1, MD_ACCESS_WRITE = 0x2;
var MD_AUTH_ANONYMOUS = 0x1, MD_AUTH_BASIC = 0x2, MD_AUTH_NT = 0x4;
var MD_ACCESS_READ = 0x1, MD_ACCESS_WRITE = 0x2, MD_ACCESS_EXECUTE = 0x4, MD_ACCESS_SCRIPT = 0x200;
var ACCESS_DRIVER = "Microsoft Access Driver (*.mdb)", SQL_DRIVER = "SQL Server";


// ***** customize the following values for your specific needs *****
// debugger;
sHost = "localhost";  // use "localhost" to carry out tasks on host where script is running
sWntDomain = "\0";  // use null string to carry out tasks against local security list
sNicInstance = "NdisLoop3";  // nicInstance name to carry out IP settings against
// use ipconfig /all |more and look for heading "Ethernet adapter <sNicInstance>:" to derive value
sIpAddressStart = "192.168.0.1";  // starting IP address for range
nIpAddressRange = 20;  // number of IPs and customers to provision starting at lIpAddressStart
sSubnetMask = "255.255.255.0";  // subnet mask in hex [i.e. 0xffffff00 = 255.255.255.0]
bAddToExisting = FALSE;  // TRUE = add, FALSE = replace existing IP settings on nicInstance
sHostingFolder = "C:\\Inetpub\\Customers";  // customer hosting parent folder
sHostingPrefix = "cust";	// prefix to use for users, folders, ftpRoot, webRoot, db and dsn
// sDbHost = "\0";  // use null string to diasble customer db and dsn provisioning
sDbHost = "C:\\Inetpub\\Customers\\DefaultContent\\db\\default.mdb";  // create access db and dsn
// sDbHost = "(local)";  // create sql db and dsn, use "(local)" when using localhost sql service
bW2ksHost = TRUE;  // denotes whether host is W2ks or Ws40
// ***** customize the following values for your specific needs *****


// connect an object variable to the desired interfaces
// debugger;
var oTcpip = WScript.CreateObject("icuHost.tcpip");
var oCustomers = WScript.CreateObject("icuHost.customers");
var oShell = WScript.CreateObject("WScript.Shell");

// stop the ftp and web services so their won't be any issues with
// file and folder locks impacting deletions or Acl setting
// debugger;
oShell.Run("%systemroot%\\system32\\net stop msftpsvc");
oShell.Run("%systemroot%\\system32\\net stop w3svc");


// call the function specified by ArgV[1]
// debugger;
switch (sTask)
{
	case "debug":
		taskDebug();
		break;
	case "create":
		taskCreate();
		break;
	case "delete":
		taskDelete();
		break;
	default:
		break;
}


// at this point you would need to reboot the host for the IP settings to take effect
// debugger;
var oShell = WScript.CreateObject("WScript.Shell");
// oShell.Run("%systemroot%\\unattend\\utils\\shutdown /l /r /t:30 \"rebooting localhost\" /y /c");
// OR for debug purposes just display what the shutdown command would have been
WScript.Echo("%systemroot%\\unattend\\utils\\shutdown /l /r /t:30 \"rebooting localhost\" /y /c");

// disconnect object variables from interfaces
// debugger;
oTcpip = null;
oCustomers = null;
oShell = null;


// exit gracefully from script
// debugger;
WScript.Quit(0);


function errHandler()
{
	WScript.Echo("Error: One or more arguments are missing!");
 	WScript.Echo("Usage: cscript hostProv.js [debug | create | delete]");
	WScript.Quit(1);
	// return 1;
}


function taskDebug()
{
	// debugger;
	
	/*
	// test reading in current NicInstanceName IP settings
	oTcpip.Host = sHost;
	oTcpip.NicInstance = sNicInstance;
	var sIpAddressRegString = oTcpip.IpAddressRegString; 
	var sSubnetMaskRegString = oTcpip.SubnetMaskRegString;
	*/
	
	// test assigning and reading back the ipAddressStart and subnetMask values
	debugger;
	oTcpip.IpAddressStart = sIpAddressStart;
	oTcpip.SubnetMask = sSubnetMask;
	sIpAddressStart = oTcpip.IpAddressStart;
	sSubnetMask = oTcpip.SubnetMask;

	// test customer creation interfaces
	oCustomers.Host = sHost;
	oCustomers.WntDomain = sWntDomain;
	oCustomers.DbHost = sDbHost;
	debugger;
	var nWebSrvNumNext = oCustomers.WebSrvNumNext;
	var nFtpSrvNumNext = oCustomers.FtpSrvNumNext;

	/*
	hr = oCustomers.UserDelete("cust101");
	// hr = oCustomers.UserCreate("cust101", "myPassword", "myFullname");
	hr = oCustomers.UserCreate("cust101", "cust101", "cust101");
	hr = oCustomers.FolderCreate(sHostingFolder);
	hr = oCustomers.FolderCreate((sHostingFolder + "\\defaultContent"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\defaultContent\\db"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\defaultContent\\index"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\defaultContent\\web"));
	hr = oCustomers.FolderDelete((sHostingFolder + "\\cust101"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\cust101"));
	var sUserOrGroup ="cust101"
	var dwFileMask = (GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | DELETE);
	var dwFolderMask = (FILE_GENERIC_READ | FILE_GENERIC_WRITE | FILE_GENERIC_EXECUTE | DELETE);
	// sUserOrGroup = "Users"
	// dwFileMask = GENERIC_ALL;
	// dwFolderMask = FILE_ALL_ACCESS;
	hr = oCustomers.AclModify((sHostingFolder + "\\cust101"), "cust101", dwFileMask, dwFolderMask, GRANT_ACCESS);
	hr = oCustomers.FolderCreate((sHostingFolder + "\\cust101\\db"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\cust101\\index"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\cust101\\web"));
	hr = oCustomers.CopyFile((sHostingFolder + "\\DefaultContent\\banner.txt"), (sHostingFolder + "\\cust101\\banner.txt"), FALSE);
	hr = oCustomers.CopyFile((sHostingFolder + "\\DefaultContent\\web\\default.htm"), (sHostingFolder + "\\cust101\\web\default.htm"), FALSE);
	// hr = oCustomers.CopyFolder((sHostingFolder + "\\DefaultContent"), (sHostingFolder + "\\cust101"), FALSE);
	unFtpSrvNumNext = oCustomers.FtpSrvNumNext;
	hr = oCustomers.FtpSrvDelete("192.168.55.101", "cust101");
	hr = oCustomers.WebSrvDelete("192.168.55.101", "cust101");
	hr = oCustomers.FtpSrvCreate("192.168.55.101", "cust101", (sHostingFolder + "\\cust101"), FALSE, (MD_ACCESS_READ | MD_ACCESS_WRITE));
	hr = oCustomers.WebSrvCreate("192.168.55.101", "cust101", (sHostingFolder + "\\cust101\\web"), (MD_AUTH_ANONYMOUS | MD_AUTH_BASIC), (MD_ACCESS_READ | MD_ACCESS_WRITE | MD_ACCESS_SCRIPT));
	*/
	hr = oCustomers.DbDelete("cust101", (sHostingFolder + "\\DefaultContent\\db"));
	hr = oCustomers.DbCreate("cust101", (sHostingFolder + "\\DefaultContent\\db"));
	// hr = oCustomers.DbDelete("cust101", "\0"));  // use sql65 default device location
	// hr = oCustomers.DbCreate("cust101", "\0");  // use sql65 default device location
	hr = oCustomers.DsnDelete("cust101", (sHostingFolder + "\\cust101\\db"));
	hr = oCustomers.DsnCreate("cust101", (sHostingFolder + "\\cust101\\db"));
	return 0;
}


function taskCreate()
{
	// create range of IP's on specified NicInstance
	// debugger;
	oTcpip.Host = sHost;
	oTcpip.NicInstance = sNicInstance;
	oTcpip.IpAddressStart = lIpAddressStart;
	oTcpip.IpAddressRange = lIpAddressRange;
	oTcpip.SubnetMask = lSubnetMask;
	oTcpip.AddToExisting = bAddToExisting;
	oTcpip.ApplyAddresses();  // apply range of IP addresses to nicInstance

	// create range of users, folders, acls, ftpRoots and webRoots
	// debugger;
	oCustomers.IpAddressStart = lIpAddressStart;
	oCustomers.IpAddressRange = lIpAddressRange;
	oCustomers.SubnetMask = lSubnetMask;
	oCustomers.Host = sHost;
	oCustomers.WntDomain = sWntDomain;
	oCustomers.HostingFolder = sHostingFolder;
	oCustomers.HostingPrefix = sHostingPrefix;
	oCustomers.DbHost = sDbHost;
	// Create customer folder and default content folders just in case this has not already
	// been done.  If it has FolderCreate will not do anything to harm existing folders.
	hr = oCustomers.FolderCreate(sHostingFolder);
	hr = oCustomers.FolderCreate((sHostingFolder + "\\DefaultContent"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\DefaultContent\\db"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\DefaultContent\\index"));
	hr = oCustomers.FolderCreate((sHostingFolder + "\\DefaultContent\\web"));
	oCustomers.CustomersCreate();  // create range of customer hosting setups

	return 0;
}


function taskDelete()
{
	// delete range of users, folders, acls, ftpRootsa and webRoots
	// debugger;
	oCustomers.IpAddressStart = lIpAddressStart;
	oCustomers.IpAddressRange = lIpAddressRange;
	oCustomers.SubnetMask = lSubnetMask;
	oCustomers.Host = sHost;
	oCustomers.WntDomain = sWntDomain;
	oCustomers.HostingFolder = sHostingFolder;
	oCustomers.HostingPrefix = sHostingPrefix;
	oCustomers.DbHost = sDbHost;
	oCustomers.CustomersDelete();  // delete range of customer hosting setups

	// restore NicInstance IP's using literal string values
	// a replacement of the current settings is implied
	// debugger;
	oTcpip.Host = sHost;
	oTcpip.NicInstance = sNicInstance;
	oTcpip.IpAddressRegString = "0.0.0.0";  // using all zeros switches nicInstance to dhcp addressing
	oTcpip.SubnetMaskRegString = "0.0.0.0";
	// oTcpip.IpAddressRegString = "192.168.55.102";  // switch nicInstance to single static ip
	// oTcpip.SubnetMaskRegString = "255.255.255.0";

	return 0;
}


/*
    The End
*/
