// Tcpip.cpp : Implementation of CTcpip
#include "stdafx.h"
#include "IcuHost.h"
#include "Tcpip.h"

#include <lmerr.h>  // used by GetLastErrorData method

/////////////////////////////////////////////////////////////////////////////
// CTcpip

STDMETHODIMP CTcpip::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ITcpip
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


STDMETHODIMP CTcpip::get_AddToExisting(BOOL *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_bAddToExisting;
	return S_OK;
}


STDMETHODIMP CTcpip::put_AddToExisting(BOOL newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_bAddToExisting = newVal;
	return S_OK;
}


STDMETHODIMP CTcpip::get_Host(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	// TODO: Add your implementation code here
	*pVal = m_sHost;
	return S_OK;
}


STDMETHODIMP CTcpip::put_Host(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sHost = newVal;
	if (m_sHost.Length() && _tcscmp(_T("localhost"), m_sHost))
		m_bRemoteHost = TRUE;
	else
	{
		m_bRemoteHost = FALSE;
		_TCHAR aValue[MAX_CB_NBHOSTNAME] = {_T("\0")};
		DWORD dwRet = GetEnvironmentVariable(_T("COMPUTERNAME"), aValue, MAX_CB_NBHOSTNAME);
		if (dwRet) 
			m_sHost = aValue;
	}
	
	return S_OK;
}


STDMETHODIMP CTcpip::get_IpAddressRange(DWORD *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_dwIpAddressRange;
	return S_OK;
}


STDMETHODIMP CTcpip::put_IpAddressRange(DWORD newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_dwIpAddressRange = newVal;
	return S_OK;
}


STDMETHODIMP CTcpip::get_IpAddressRegString(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	BOOL bRet = FALSE;
	HKEY hKey = HKEY_LOCAL_MACHINE, hSubKey = 0;

	// todo:  if (bW2ksHost)... then set Ip's via different approach
	if (m_bRemoteHost)
	{
		CComBSTR sUncHost(_T("\\\\")); sUncHost.Append(m_sHost);
		long lr = RegConnectRegistry(sUncHost, hKey, &hKey);
		if (ERROR_SUCCESS != lr)
			return E_FAIL;
	}
	
	CComBSTR sSubKey(_T("System\\CurrentControlSet\\Services\\")); 
	sSubKey.Append(m_sNicInstance); sSubKey.Append(_T("\\Parameters\\Tcpip"));
	
	long lr = RegOpenKeyEx(hKey, sSubKey, 0, KEY_QUERY_VALUE, &hSubKey);
	if (lr) 
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}

	DWORD dwValueType = REG_MULTI_SZ;
	DWORD cbValue = MAX_CB_REG_MULTI_SZ;
	unsigned char aValue[MAX_CB_REG_MULTI_SZ] = {"\0"};
	
	lr = RegQueryValueEx(hSubKey, _T("IpAddress"), 0, &dwValueType, &aValue[0], &cbValue);
	if (lr) 
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}	

	m_sIpAddressRegString.Empty();
	cbValue -= (2 * sizeof(_TCHAR)); // compensate for two nulls that terminate a reg_multi_sz
	for (int nIndex = 0; nIndex < (int)cbValue; nIndex += sizeof(_TCHAR))
	{
		if (aValue[nIndex])
			m_sIpAddressRegString.Append((BSTR)&aValue[nIndex], 1);
		else
			m_sIpAddressRegString.Append(_T(" "));
	}

	bRet = CloseKey(hKey, hSubKey);

	if (pVal)
		*pVal = m_sIpAddressRegString;
	else
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		return E_FAIL;
	}
		
	return S_OK;
}


STDMETHODIMP CTcpip::put_IpAddressRegString(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (newVal)
		m_sIpAddressRegString = newVal;
	else
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		// do not return since this is most likely a call from ApplyAddresses
		// return E_FAIL;
	}
	
	BOOL bRet = FALSE;
	HKEY hKey = HKEY_LOCAL_MACHINE, hSubKey = 0;

	if (m_bRemoteHost)
	{
		CComBSTR sUncHost(_T("\\\\")); sUncHost.Append(m_sHost);
		long lr = RegConnectRegistry(sUncHost, hKey, &hKey);
		if (ERROR_SUCCESS != lr)
			return E_FAIL;
	}
	
	CComBSTR sSubKey(_T("System\\CurrentControlSet\\Services\\")); 
	sSubKey.Append(m_sNicInstance); sSubKey.Append(_T("\\Parameters\\Tcpip"));
	
	long lr = RegOpenKeyEx(hKey, sSubKey, 0, KEY_SET_VALUE, &hSubKey);
	if (lr) 
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}

	DWORD dwValueType = REG_MULTI_SZ;
	DWORD cbValue = m_sIpAddressRegString.Length();
	unsigned char aValue[MAX_CB_REG_MULTI_SZ] = {"\0"};

	for (int nIndex = 0; nIndex < (int)cbValue; nIndex++)
	{
		if (_T(' ') != m_sIpAddressRegString.m_str[nIndex])
		{
			_tcsncpy((BSTR)&aValue[nIndex*sizeof(_TCHAR)], &m_sIpAddressRegString.m_str[nIndex], 1);
			aValue[nIndex*sizeof(_TCHAR)+1] = '\0';
		}
		else
		{
			// separate reg_multi_sz entries will a null
			aValue[nIndex*sizeof(_TCHAR)] = '\0'; 
		}
	}

	// terminate reg_multi_sz with extra null
	aValue[nIndex * sizeof(_TCHAR)] = '\0'; 

	// adjust buffer count of bytes to account for unicode value
	cbValue *= sizeof(_TCHAR);

	lr = RegSetValueEx(hSubKey, _T("IpAddress"), 0, dwValueType, &aValue[0], cbValue);
	if (lr) 
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}	

	//  configure enableDhcp value appropriately
	dwValueType = REG_DWORD;
	cbValue = sizeof(DWORD);  // dword is four bytes
	for (nIndex = 0; nIndex < (int)cbValue; nIndex++)
		aValue[nIndex] = 0x0;  // zero out the dword bytes

	if (_tcscmp(_T("0.0.0.0"), m_sIpAddressRegString))
		aValue[0] = 0x0;  // set the dword value to disable dhcp addressing
	else
		aValue[0] = 0x1;  // set the dword value to enable dhcp addressing
	
	lr = RegSetValueEx(hSubKey, _T("EnableDHCP"), 0, dwValueType, &aValue[0], cbValue);
	if (lr) 
	{
		GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}	
	
	bRet = CloseKey(hKey, hSubKey);

	return S_OK;
}


STDMETHODIMP CTcpip::get_IpAddressStart(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	DWORD dwIpAddress = m_dwIpAddressStart, dwIpAddressByte = 0;
	CComBSTR sIpAddress;
	_TCHAR aByte[sizeof(_TCHAR) * MAX_BYTES_PER_OCTET] = {_T("\0")};

	for (int nByte = 4; nByte > 0; nByte--)
	{
		dwIpAddressByte = (dwIpAddress & ((1<<8*nByte) - (1<<8*(nByte-1)))) >> 8*(nByte-1);
		wsprintf(aByte, _T("%d"), dwIpAddressByte);
		sIpAddress.Append(aByte);
		if (1 != nByte)
			sIpAddress.Append(_T("."));
	}

	*pVal = sIpAddress;
	return S_OK;
}


STDMETHODIMP CTcpip::put_IpAddressStart(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	CString sIpAddress = newVal;
	DWORD dwIpAddressByte = 0;
	int nShift = 24;
	m_dwIpAddressStart = 0;

	for (int nByte = 4; nByte > 0; nByte--)
	{
		if (1 != nByte)
		{
			dwIpAddressByte = _tcstoul(sIpAddress.Left(sIpAddress.FindOneOf(_T("."))), NULL, 10) << nShift;
			m_dwIpAddressStart += dwIpAddressByte;
			sIpAddress = sIpAddress.Mid(sIpAddress.FindOneOf(_T(".")) + 1);
			nShift -= 8;
		}
		else
		{
			dwIpAddressByte = _tcstoul(sIpAddress, NULL, 10);
			m_dwIpAddressStart += dwIpAddressByte;
		}
	}

	return S_OK;
}


STDMETHODIMP CTcpip::get_LastErrorDesc(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (pVal)
		*pVal = m_sLastErrorDesc;

	return S_OK;
}


STDMETHODIMP CTcpip::get_LastErrorNum(LONG *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (pVal)
		*pVal = m_lLastErrorNum;

	return S_OK;
}


STDMETHODIMP CTcpip::get_NicInstance(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sNicInstance;
	return S_OK;
}


STDMETHODIMP CTcpip::put_NicInstance(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sNicInstance = newVal;
	return S_OK;
}


STDMETHODIMP CTcpip::get_SubnetMask(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	DWORD dwSubnetMask = m_dwSubnetMask, dwSubnetMaskByte = 0;
	CComBSTR sSubnetMask;
	_TCHAR aByte[sizeof(_TCHAR) * MAX_BYTES_PER_OCTET] = {_T("\0")};

	for (int nByte = 4; nByte > 0; nByte--)
	{
		dwSubnetMaskByte = (dwSubnetMask & ((1<<8*nByte) - (1<<8*(nByte-1)))) >> 8*(nByte-1);
		wsprintf(aByte, _T("%d"), dwSubnetMaskByte);
		sSubnetMask.Append(aByte);
		if (1 != nByte)
			sSubnetMask.Append(_T("."));
	}

	*pVal = sSubnetMask;
	return S_OK;
}


STDMETHODIMP CTcpip::put_SubnetMask(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	CString sSubnetMask = newVal;
	DWORD dwSubnetMaskByte = 0;
	int nShift = 24;
	m_dwSubnetMask = 0;

	for (int nByte = 4; nByte > 0; nByte--)
	{
		if (1 != nByte)
		{
			dwSubnetMaskByte = _tcstoul(sSubnetMask.Left(sSubnetMask.FindOneOf(_T("."))), NULL, 10) << nShift;
			m_dwSubnetMask += dwSubnetMaskByte;
			sSubnetMask = sSubnetMask.Mid(sSubnetMask.FindOneOf(_T(".")) + 1);
			nShift -= 8;
		}
		else
		{
			dwSubnetMaskByte = _tcstoul(sSubnetMask, NULL, 10);
			m_dwSubnetMask += dwSubnetMaskByte;
		}
	}

	return S_OK;
}


STDMETHODIMP CTcpip::get_SubnetMaskRegString(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	BOOL bRet = FALSE;
	HKEY hKey = HKEY_LOCAL_MACHINE, hSubKey = 0;

	if (m_bRemoteHost)
	{
		CComBSTR sUncHost(_T("\\\\")); sUncHost.Append(m_sHost);
		long lr = RegConnectRegistry(sUncHost, hKey, &hKey);
		if (ERROR_SUCCESS != lr)
			return E_FAIL;
	}
	
	CComBSTR sSubKey(_T("System\\CurrentControlSet\\Services\\")); 
	sSubKey.Append(m_sNicInstance); sSubKey.Append(_T("\\Parameters\\Tcpip"));
	
	long lr = RegOpenKeyEx(hKey, sSubKey, 0, KEY_QUERY_VALUE, &hSubKey);
	if (lr) 
	{
		get_LastErrorDesc(NULL);
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}

	DWORD dwValueType = REG_MULTI_SZ;
	DWORD cbValue = MAX_CB_REG_MULTI_SZ;
	unsigned char aValue[MAX_CB_REG_MULTI_SZ] = {"\0"};

	lr = RegQueryValueEx(hSubKey, _T("SubnetMask"), 0, &dwValueType, &aValue[0], &cbValue);
	if (lr) 
	{
		get_LastErrorDesc(NULL);
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}	

	m_sSubnetMaskRegString.Empty();
	cbValue -= (2 * sizeof(_TCHAR)); // compensate for two nulls that terminate a reg_multi_sz
	for (int nIndex = 0; nIndex < (int)cbValue; nIndex += sizeof(_TCHAR))
	{
		if (aValue[nIndex])
			m_sSubnetMaskRegString.Append((BSTR)&aValue[nIndex], 1);
		else
			m_sSubnetMaskRegString.Append(_T(" "));
	}
	
	bRet = CloseKey(hKey, hSubKey);

	if (pVal)
		*pVal = m_sSubnetMaskRegString;
	else
	{
		get_LastErrorDesc(NULL);
		return E_FAIL;
	}
	
	return S_OK;
}


STDMETHODIMP CTcpip::put_SubnetMaskRegString(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (newVal)
		m_sSubnetMaskRegString = newVal;
	else
	{
		get_LastErrorDesc(NULL);
		// do not return since this is most likely a call from ApplyAddresses
		// return E_FAIL;
	}
	
	BOOL bRet = FALSE;
	HKEY hKey = HKEY_LOCAL_MACHINE, hSubKey = 0;

	if (m_bRemoteHost)
	{
		CComBSTR sUncHost(_T("\\\\")); sUncHost.Append(m_sHost);
		long lr = RegConnectRegistry(sUncHost, hKey, &hKey);
		if (ERROR_SUCCESS != lr)
			return E_FAIL;
	}
	
	CComBSTR sSubKey(_T("System\\CurrentControlSet\\Services\\")); 
	sSubKey.Append(m_sNicInstance); sSubKey.Append(_T("\\Parameters\\Tcpip"));
	
	long lr = RegOpenKeyEx(hKey, sSubKey, 0, KEY_SET_VALUE, &hSubKey);
	if (lr) 
	{
		get_LastErrorDesc(NULL);
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}

	DWORD dwValueType = REG_MULTI_SZ;
	DWORD cbValue = m_sSubnetMaskRegString.Length();
	unsigned char aValue[MAX_CB_REG_MULTI_SZ] = {"\0"};
	
	for (int nIndex = 0; nIndex < (int)cbValue; nIndex++)
	{
		if (_T(' ') != m_sSubnetMaskRegString.m_str[nIndex])
		{
			_tcsncpy((BSTR)&aValue[nIndex*sizeof(_TCHAR)], &m_sSubnetMaskRegString.m_str[nIndex], 1);
			aValue[nIndex*sizeof(_TCHAR)+1] = '\0';
		}
		else
		{
			// separate reg_multi_sz entries will a null
			aValue[nIndex*sizeof(_TCHAR)] = '\0';
		}
	}

	// terminate reg_multi_sz with extra null
	aValue[nIndex * sizeof(_TCHAR)] = '\0'; 

	// adjust buffer count of bytes to account for unicode value
	cbValue *= sizeof(_TCHAR);

	lr = RegSetValueEx(hSubKey, _T("SubnetMask"), 0, dwValueType, &aValue[0], cbValue);
	if (lr) 
	{
		get_LastErrorDesc(NULL);
		bRet = CloseKey(hKey, hSubKey);
		return E_FAIL;
	}	
	
	bRet = CloseKey(hKey, hSubKey);

	return S_OK;
}


STDMETHODIMP CTcpip::ApplyAddresses()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (m_bAddToExisting)
	{
		HRESULT hr = get_IpAddressRegString(NULL);
		hr = get_SubnetMaskRegString(NULL);
	}
	else
	{
		m_sIpAddressRegString.Empty();
		m_sSubnetMaskRegString.Empty();
	}

	DWORD dwIpAddress = 0;
	__int64 dwIpAddressByte = 0, dwSubnetMaskByte = 0;
	HRESULT hr;
	
	_TCHAR aByte[MAX_BYTES_PER_OCTET + 1] = {_T("\0")};
	dwIpAddress = m_dwIpAddressStart;

	for (int nIndex = 0; nIndex < (int)m_dwIpAddressRange; nIndex++, dwIpAddress++)
	{
		for (int nByte = 4; nByte > 0; nByte--)
		{
			dwIpAddressByte = (dwIpAddress & ((1<<8*nByte) - (1<<8*(nByte-1)))) >> 8*(nByte-1);
			dwSubnetMaskByte = (m_dwSubnetMask & ((1<<8*nByte) - (1<<8*(nByte-1)))) >> 8*(nByte-1);
			wsprintf(aByte, _T("%d"), dwIpAddressByte);
			m_sIpAddressRegString.Append(aByte);
	
			wsprintf(aByte, _T("%d"), dwSubnetMaskByte);
			m_sSubnetMaskRegString.Append(aByte);
			
			if (1 != nByte)
			{
				m_sIpAddressRegString.Append(_T("."));
				m_sSubnetMaskRegString.Append(_T("."));
			}
			else
			{
				m_sIpAddressRegString.Append(_T(" "));
				m_sSubnetMaskRegString.Append(_T(" "));
			}
		}
	}

	hr = put_IpAddressRegString(NULL);
	if (FAILED(hr))
		return E_FAIL;
	hr = put_SubnetMaskRegString(NULL);
	if (FAILED(hr))
		return E_FAIL;
	
	return S_OK;
}


BOOL CTcpip::CloseKey(HKEY hKey, HKEY hSubKey)
{	
	RegCloseKey(hSubKey);
	RegCloseKey(hKey);
	if (m_bRemoteHost)
	{
		CComBSTR sUncHostIpc(_T("\\\\")); 
		sUncHostIpc.Append(m_sHost); sUncHostIpc.Append(_T("\\ipc$"));

		// implicit linkink with mpr.lib
		DWORD dwRet = WNetCancelConnection2(sUncHostIpc, 0, FALSE);
	}
	return S_OK;
}


STDMETHODIMP CTcpip::GetLastErrorData(LONG lLastErrorNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

    HMODULE hModule = NULL;  // default to standard system dlls as source for api returned errors
    LPTSTR pszLastErrorDesc = NULL;
    UINT cbLastErrorDesc = 0;
	_TCHAR szText[MAX_CB_BUFFER] = {_T("\0")}, szCaption[MAX_CB_BUFFER] = {_T("\0")};

	if (lLastErrorNum)  // check if caller passed a lLastErrorNum using a HRESULT value
		m_lLastErrorNum = lLastErrorNum;  // assign value also to the member variable 										 
	else    // caller did not pass a lLastErrorNum using a HRESULT value
	{
		lLastErrorNum = GetLastError();  // acquire the lLastErrorNum value from GetLastError() api
		m_lLastErrorNum = lLastErrorNum;  // assign value also to the member variable
	}

    DWORD dwFormatFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER |
						  FORMAT_MESSAGE_IGNORE_INSERTS |
						  FORMAT_MESSAGE_FROM_SYSTEM ;

    // if lLastErrorNum is in the network range, load the message source
    if (lLastErrorNum >= NERR_BASE && lLastErrorNum <= MAX_NERR) 
	{
        hModule = LoadLibraryEx(_T("netmsg.dll"), NULL, LOAD_LIBRARY_AS_DATAFILE);
        if (hModule)
			dwFormatFlags |= FORMAT_MESSAGE_FROM_HMODULE;
    }

    // use FormatMessage() to acquire the text information for a given error number
	cbLastErrorDesc = FormatMessage(dwFormatFlags, hModule, lLastErrorNum,
									MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // use default language
								   (LPTSTR)&pszLastErrorDesc, 0, NULL);
    if (cbLastErrorDesc)
    {
        // if we are in debug mode then output error message to stderr
		#ifdef _DEBUG
			DWORD dwBytesWritten = 0;
			WriteFile(GetStdHandle(STD_ERROR_HANDLE), pszLastErrorDesc, 
					  (sizeof(_TCHAR) * cbLastErrorDesc), &dwBytesWritten, NULL);
		#endif

		// if this is a console application then output the error description to stdout
		// printf("errNum=%lu, errDesc=\"%s\"\n", lLastErrorNum, pszLastErrorDesc);

		// if this is a windows application then output the error description to a MessageBox
		// wsprintf(szText, _T("Api or COM interface call produced error %lu,\n%s."), lLastErrorNum, pszLastErrorDesc);
		// wsprintf(szCaption, _T("GetLastErrorData"));
		// MessageBox(NULL, szText, szCaption,  MB_OK | MB_ICONINFORMATION);
       
        // assign error values to member variable and free the buffer allocated by the api call
		m_sLastErrorDesc.Append(_ltot(lLastErrorNum, szText, 10));
		m_sLastErrorDesc.Append(_T(", ")); m_sLastErrorDesc.Append(pszLastErrorDesc);
        LocalFree(pszLastErrorDesc);
    }

    // if we loaded a message source, unload it
    if (hModule != NULL)
		FreeLibrary(hModule);

	return S_OK;
}

STDMETHODIMP CTcpip::get_bW2ksHost(BOOL *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	// TODO: Add your implementation code here

	return S_OK;
}

STDMETHODIMP CTcpip::put_bW2ksHost(BOOL newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	// TODO: Add your implementation code here

	return S_OK;
}
