VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "registry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private g_sErrInfo As String, g_bCancel As Boolean

Public Event Status(sStatusInfo As String, ByRef bCancel As Boolean)

Public Property Get ErrInfo() As String
    ErrInfo = g_sErrInfo
End Property

Public Function CheckForKey(hKey As Long, sSubKey As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        CheckForKey = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            CheckForKey = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0&, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CheckForKey = lRet
        Exit Function
    End If
    
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CheckForKey = 0
        
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CheckForKey = 1
End Function

Public Function CheckForValue(hKey As Long, sSubKey As String, sValueName As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValueName = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        CheckForValue = 1
        Exit Function
    End If
        
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lValueLen As Long, aValue() As Byte, iIndex As Integer, lValueType As Long
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            CheckForValue = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CheckForValue = lRet
        Exit Function
    End If
    
    '// since we don't know the lValueType yet allocate the maximum space _
        required in aValue by a returned value
    lValueLen = MAX_REG_STRING_LEN - 1
    ReDim aValue(lValueLen)
    For iIndex = 0 To (lValueLen - 1) '// set all bytes to '*'
        aValue(iIndex) = 47 '// ascii character code for '*'
    Next '// iIndex
    aValue(iIndex) = 0 '// add null terminator
    lValueLen = lValueLen + 1 '// add 1 for null terminator
    
    lRet = RegQueryValueEx(phSubKey, sValueName, 0&, lValueType, aValue(0), lValueLen)
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to query to registry value, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to query to registry value."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CheckForValue = lRet
        Exit Function
    End If
    
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CheckForValue = 0
            
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CheckForValue = 1
End Function

Public Function CreateKey(hKey As Long, sSubKey As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        CreateKey = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lDisposition As Long
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            CreateKey = lRet
            Exit Function
        End If
    End If
    
    lRet = RegCreateKeyEx(phKey, sSubKey, 0&, vbNullString, _
                          REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, _
                          0&, phSubKey, lDisposition)
                          
    If (lRet) Then
        g_sErrInfo = "Unable to create registry subkey."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CreateKey = lRet
        Exit Function
    Else
        If (lDisposition = REG_CREATED_NEW_KEY) Then
            g_sErrInfo = "REG_CREATED_NEW_KEY"
        End If
        
        If (lDisposition = REG_OPENED_EXISTING_KEY) Then
            g_sErrInfo = "REG_OPENED_EXISTING_KEY) "
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CreateKey = 0
    End If
        
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CreateKey = 1
End Function

Public Function CreateValue(hKey As Long, sSubKey As String, sValueName As String, _
                            lValueType As Long, vValue As Variant, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValueName = "" Or IsEmpty(lValueType) Or IsEmpty(vValue)) Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        CreateValue = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        sValueTemp As String, lValueLen As Long, aValue() As Byte, iIndex As Integer, _
        lValueTemp As Long, iStartByte As Integer, iBit As Integer, _
        iByte As Integer, iByteBit As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName) '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            CreateValue = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CreateValue = lRet
        Exit Function
    End If
    
    Select Case (lValueType)
    Case REG_SZ, REG_EXPAND_SZ, REG_MULTI_SZ '// assume vValue passed as string
        sValueTemp = CStr(vValue) '// ensure vValue is cast as a string
        lValueLen = Len(sValueTemp)
        
        If (lValueType = REG_MULTI_SZ) Then
            If (InStr((lValueLen), sValueTemp, vbNullChar, vbBinaryCompare) = 0) Then
                sValueTemp = sValueTemp & vbNullChar
                lValueLen = lValueLen + 1
            End If
        End If
        
        ReDim aValue(lValueLen)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = Asc(Mid(sValueTemp, (iIndex + 1), 1))
        Next '// iIndex
        
        aValue(iIndex) = 0 '// add null terminator
        lValueLen = lValueLen + 1 '// add 1 for null terminator
        
    Case REG_BINARY '// assume vValue passed as byte array
        lValueLen = UBound(vValue) + 1
        ReDim aValue(lValueLen)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = vValue(iIndex) '// set bytes arrays equal
        Next '// iIndex
        
        aValue(iIndex) = 0 '// set null terminator
        
    Case REG_DWORD
        lValueTemp = CLng(vValue)
        lValueLen = MAX_REG_DWORD_LEN
        ReDim aValue(lValueLen - 1)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = 0 '// set bytes to 0
        Next '// iIndex
        
        If (lValueTemp And &H80000000) Then '// To work around lack of unsigned long data type
            aValue(lValueLen - 1) = &H80
            iStartByte = lValueLen - 2
            iBit = 23
            lValueTemp = lValueTemp - &H80000000 '// adjust value for calculation of remaining bytes
        Else
            iStartByte = lValueLen - 1
            iBit = 31
        End If
        
        For iByte = iStartByte To 0 Step -1
            For iByteBit = 7 To 0 Step -1
                If ((lValueTemp - (2 ^ iBit)) >= 0) Then
                    aValue(iByte) = aValue(iByte) + (2 ^ iByteBit)
                    lValueTemp = lValueTemp - (2 ^ iBit)
                End If
                iBit = iBit - 1
            Next '// iByteBit
        Next '// iByte
    End Select
        
    lRet = RegSetValueEx(phSubKey, sValueName, 0&, lValueType, aValue(0), lValueLen)
                          
    If (lRet) Then
        g_sErrInfo = "Unable to create registry value."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        CreateValue = lRet
        Exit Function
    End If
    
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CreateValue = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    CreateValue = 1
End Function

Public Function DeleteKey(hKey As Long, sSubKey As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        DeleteKey = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            DeleteKey = lRet
            Exit Function
        End If
    End If
    
    lRet = RegDeleteKey(phKey, sSubKey)
                          
    If (lRet) Then
        g_sErrInfo = "Unable to delete registry subkey."
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        DeleteKey = lRet
        Exit Function
    Else
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        DeleteKey = 0
    End If
        
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    DeleteKey = 1
End Function

Public Function DeleteValue(hKey As Long, sSubKey As String, sValueName As String, _
                            Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValueName = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        DeleteValue = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        sValueTemp As String, lValueLen As Long, aValue() As Byte, iIndex As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            DeleteValue = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        DeleteValue = lRet
        Exit Function
    End If
    
    lRet = RegDeleteValue(phSubKey, sValueName)
                          
    If (lRet) Then
        g_sErrInfo = "Unable to delete registry value."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        DeleteValue = lRet
        Exit Function
    End If
    
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    DeleteValue = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    DeleteValue = 1
End Function

Private Function EnsureUncName(sName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (sName = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        EnsureUncName = 1
        Exit Function
    End If
        
    sName = Trim(sName)

    If (Left(sName, 2)) = "\\" Then
        EnsureUncName = 0
        Exit Function
    End If
        
    If (Left(sName, 1) = "\") Then
        sName = "\" & sName
        EnsureUncName = 0
        Exit Function
    End If
    
    sName = "\\" & sName
    EnsureUncName = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    EnsureUncName = 1
End Function

Function QueryAllValues(hKey As Long, sSubKey As String, caValuesFound As Collection, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or IsEmpty(caValuesFound)) Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        QueryAllValues = 1
        Exit Function
    End If
        
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lClassLen As Long, tLastWriteTime As stFileTime, sClass As String, _
        lNumSubKeys As Long, lMaxSubKeyLen As Long, lMaxClassLen As Long, _
        lNumValues As Long, lMaxValueNameLen As Long, lMaxValueLen As Long, _
        lSecDescLen As Long, tLocalFileTime As stFileTime, tLocalSystemTime As stSystemTime, _
        sDate As String * MAX_DATE_LEN, sFileDate As String, iElement As Integer, _
        iIndex As Integer, lSubKeyLen As Long, sSubSubKey As String, lValueNameLen As Long, _
        lValueLen As Long, sValueName As String, aValue() As Byte, vValue As Variant, _
        lValue As Long, lValueType As Long, oRegValue As Object, sExpandedSz As String, _
        iStartByte As Integer, iBit As Integer, iByte As Integer, iByteBit As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            QueryAllValues = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
   
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryAllValues = lRet
        Exit Function
    End If
          
    '// find out how big sClass should be
    lClassLen = 0
    lRet = RegQueryInfoKey(phSubKey, vbNullString, lClassLen, 0&, vbNull, vbNull, _
            vbNull, vbNull, vbNull, vbNull, vbNull, tLastWriteTime)
    
    If (lRet) Then
        g_sErrInfo = "Unable to query subkey info."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryAllValues = lRet
        Exit Function
    End If
   
    lClassLen = lClassLen + 1 '// add room for null terminator.
    sClass = String(lClassLen, "*") '// set all bytes to '*'

    '// now query all the information about the key, including max lengths
    lRet = RegQueryInfoKey(phSubKey, sClass, lClassLen, 0&, lNumSubKeys, lMaxSubKeyLen, _
           lMaxClassLen, lNumValues, lMaxValueNameLen, lMaxValueLen, lSecDescLen, tLastWriteTime)
         
    If (lRet) Then
        g_sErrInfo = "Unable to query subkey info."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryAllValues = lRet
        Exit Function
    End If
   
    '// sClass needs the null terminator stripped off
    sClass = Left(sClass, lClassLen)
   
    '// add 1 for null terminators
    lMaxSubKeyLen = lMaxSubKeyLen + 1
    lMaxValueNameLen = lMaxValueNameLen + 1
    lMaxValueLen = lMaxValueLen + 1
    lMaxClassLen = lMaxClassLen + 1
    
    '// convert the last write time to something printable
    lRet = FileTimeToLocalFileTime(tLastWriteTime, tLocalFileTime)
    lRet = FileTimeToSystemTime(tLocalFileTime, tLocalSystemTime)
    lRet = GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, tLocalSystemTime, _
                        vbNullString, sDate, (MAX_DATE_LEN - 1))
    If (lRet) Then
        sFileDate = Left(sDate, lRet - 1)
    Else
        g_sErrInfo = "Unable to resolve registry last write time."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryAllValues = 1
        Exit Function
    End If
        
    If (lNumSubKeys) Then '// lets walk through the list of subkeys in the specified subkey
        iElement = 0
        While (iElement < lNumSubKeys)
            lSubKeyLen = lMaxSubKeyLen
            lClassLen = lMaxClassLen
            sSubSubKey = String(lSubKeyLen, "*") '// set all bytes to '*'
            sClass = String(lClassLen, "*") '// set all bytes to '*'
              
            lRet = RegEnumKeyEx(phSubKey, iElement, sSubSubKey, lSubKeyLen, _
                                0&, sClass, lClassLen, tLastWriteTime)
         
            If (lRet) Then
                If (lRet = ERROR_NO_MORE_ITEMS) Then
                    g_sErrInfo = "RegEnumKeyEx failed with, ERROR_NO_MORE_ITEMS"
                Else
                    g_sErrInfo = "RegEnumKeyEx failed with Error code " & lRet
                End If
                Call RegCloseKey(phSubKey)
                Call RegCloseKey(phKey)
                If (bRemoteReg) Then
                    Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
                End If
                QueryAllValues = 1
                Exit Function
            End If
         
            '// strip off the extra characters including the C null terminator
            sSubSubKey = Left(sSubSubKey, lSubKeyLen)
            sClass = Left(sClass, lClassLen)
            
            '// convert the last write time to something printable
            lRet = FileTimeToLocalFileTime(tLastWriteTime, tLocalFileTime)
            lRet = FileTimeToSystemTime(tLocalFileTime, tLocalSystemTime)
            lRet = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, tLocalSystemTime, _
                                vbNullString, sDate, 255)
            If (lRet) Then
                sFileDate = Left(sDate, lRet - 1)
            Else
                g_sErrInfo = "Unable to resolve registry subkey last write time."
                Call RegCloseKey(phSubKey)
                Call RegCloseKey(phKey)
                If (bRemoteReg) Then
                    Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
                End If
                QueryAllValues = 1
                Exit Function
            End If
            
            iElement = iElement + 1
        Wend '// Enum next SubKey
    
    Else '// lNumSubKeys = 0
        '// We may want to return g_sErrInfo with some information on this event _
            but since we are not looking to build a subkey collection with this _
            interface then lets save that for a QueryAllKeys interface.
    
        '// Let's populate the error information string but not return since _
            this result doesn't indicate a failure.
        '// g_sErrInfo = "There were no subkeys found."
        
    End If
   
    '// lets walk through the list of values in the specified subkey
    If (lNumValues) Then
        '// get some buffer space for Win32 to return these strings
        ReDim aValue(lMaxValueLen) '// set max size requirement of byte array
        iElement = 0
        While (iElement < lNumValues)
            lValueNameLen = lMaxValueNameLen
            lValueLen = lMaxValueLen
            sValueName = String(lValueNameLen, "*") '// set all bytes to '*'
            For iIndex = 0 To (lValueLen - 1) '// set all bytes to '*'
                aValue(iIndex) = 47 '// ascii character code for '*'
            Next '// iIndex
            aValue(iIndex) = 0 '// add null terminator
                       
            lRet = RegEnumValue(phSubKey, iElement, sValueName, lValueNameLen, _
                                0&, lValueType, aValue(0), lValueLen)
         
            If (lRet) Then
                If (lRet = ERROR_NO_MORE_ITEMS) Then
                    g_sErrInfo = "RegEnumValue failed with, ERROR_NO_MORE_ITEMS"
                Else
                    g_sErrInfo = "RegEnumValue failed with Error code " & lRet
                End If
                Call RegCloseKey(phSubKey)
                Call RegCloseKey(phKey)
                If (bRemoteReg) Then
                    Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
                End If
                QueryAllValues = 1
                Exit Function
            End If
         
            '// strip off the extra characters including the null terminator
            sValueName = Left(sValueName, lValueNameLen)
                
            Select Case (lValueType)
            Case REG_SZ, REG_EXPAND_SZ, REG_MULTI_SZ
                vValue = ""
                For iIndex = 0 To (lValueLen - 2)
                    vValue = vValue & Chr(aValue(iIndex))
                Next iIndex
                
                If (lValueType = REG_EXPAND_SZ) Then
                    lValueLen = ExpandEnvironmentStrings(vValue, vbNullString, 0&)
                    sExpandedSz = String(lValueLen, "*") '// set all bytes to '*'
                    lValueLen = ExpandEnvironmentStrings(vValue, sExpandedSz, lValueLen)
                    
                    '// lValueLen returned is incorrect due to a bug in NT 4.0 _
                        the next 2 lines work around this bug
                    lValueLen = InStr(1, sExpandedSz, vbNullChar, vbBinaryCompare)
                    vValue = Left(sExpandedSz, lValueLen - 1)
                End If
                
            Case REG_BINARY '// assume vValue passed as byte array
                ReDim vValue(lValueLen - 1)
                For iIndex = 0 To (lValueLen - 1)
                    vValue(iIndex) = aValue(iIndex) '// set bytes arrays equal
                Next '// iIndex
                
            Case REG_DWORD
                vValue = 0&
                
                If (aValue(lValueLen - 1) And &H80) Then '// To work around lack of unsigned long data type
                    vValue = &H80000000
                    iStartByte = lValueLen - 2
                    iBit = 23
                Else
                    iStartByte = lValueLen - 1
                    iBit = 31
                End If
                
                For iByte = iStartByte To 0 Step -1
                    For iByteBit = 7 To 0 Step -1
                        If ((aValue(iByte) - (2 ^ iByteBit)) >= 0) Then
                            If (iStartByte = 2) Then '// i.e. creating a negative long
                                vValue = vValue - (2 ^ iBit)
                            Else '// i.e. creating a positive long
                                vValue = vValue + (2 ^ iBit)
                            End If
                            aValue(iByte) = aValue(iByte) - (2 ^ iByteBit)
                        End If
                        iBit = iBit - 1
                    Next '// iByteBit
                Next '// iByte
            End Select
                        
            '// assign data to collection array parameter
            Set oRegValue = CreateObject("icuRegMta.regValue")
            oRegValue.ValueName = sValueName
            oRegValue.ValueType = lValueType
            oRegValue.Value = vValue
            oRegValue.Comment = "Value " & iElement & " in " & sSubKey & "."
            Call caValuesFound.Add(oRegValue)
            Set oRegValue = Nothing '// free object
            
            iElement = iElement + 1
        Wend
    
    Else '// lNumValues = 0
        '// Let's populate the error information string but not return since _
            this result doesn't indicate a failure.
        g_sErrInfo = "There were no subkey values found."
        
    End If
        
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    QueryAllValues = 0 '// return success
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    QueryAllValues = 1
End Function

Public Function QueryKey(hKey As Long, sSubKey As String, sValue As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        QueryKey = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lValueLen As Long, aValue() As Byte, iIndex As Integer, lValueType As Long
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            QueryKey = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryKey = lRet
        Exit Function
    End If
     
    lValueLen = MAX_REG_STRING_LEN - 1
    ReDim aValue(lValueLen)
    For iIndex = 0 To (lValueLen - 1)
        aValue(iIndex) = 47 '// ascii character code for '*'
    Next '// iIndex
    
    aValue(iIndex) = 0 '// add null terminator
    lValueLen = lValueLen + 1 '// add 1 to account for null terminator
    
    lRet = RegQueryValueEx(phSubKey, vbNullString, 0&, lValueType, aValue(0), lValueLen)

    If (lRet) Then
        g_sErrInfo = "Unable to Query Key Value."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryKey = lRet
        Exit Function
    End If
    
    sValue = ""
    For iIndex = 0 To (lValueLen - 2)
        sValue = sValue & Chr(aValue(iIndex))
    Next iIndex
    
    '// sValue = Left(sValueTemp, lValueLen)
    QueryKey = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    QueryKey = 1
End Function

Public Function QueryValue(hKey As Long, sSubKey As String, sValueName As String, vValue As Variant, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValueName = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        QueryValue = 1
        Exit Function
    End If
        
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lValueLen As Long, aValue() As Byte, iIndex As Integer, lValueType As Long, _
        sExpandedSz As String, iStartByte As Integer, iBit As Integer, _
        iByte As Integer, iByteBit As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            QueryValue = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryValue = lRet
        Exit Function
    End If
    
    '// since we don't know the lValueType yet allocate the maximum space _
        required in aValue by a returned value
    lValueLen = MAX_REG_STRING_LEN - 1
    ReDim aValue(lValueLen)
    For iIndex = 0 To (lValueLen - 1) '// set all bytes to '*'
        aValue(iIndex) = 47 '// ascii character code for '*'
    Next '// iIndex
    aValue(iIndex) = 0 '// add null terminator
    lValueLen = lValueLen + 1 '// add 1 for null terminator
    
    lRet = RegQueryValueEx(phSubKey, sValueName, 0&, lValueType, aValue(0), lValueLen)
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to query to registry value, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to query to registry value."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        QueryValue = lRet
        Exit Function
    End If
    
    Select Case (lValueType)
    Case REG_SZ, REG_EXPAND_SZ, REG_MULTI_SZ
        vValue = ""
        For iIndex = 0 To (lValueLen - 2)
            vValue = vValue & Chr(aValue(iIndex))
        Next iIndex
        
        If (lValueType = REG_EXPAND_SZ) Then
            lValueLen = ExpandEnvironmentStrings(vValue, vbNullString, 0&)
            sExpandedSz = String(lValueLen, "*") '// set all bytes to '*'
            lValueLen = ExpandEnvironmentStrings(vValue, sExpandedSz, lValueLen)
            
            '// lValueLen returned is incorrect due to a bug in NT 4.0 _
                the next 2 lines work around this bug
            lValueLen = InStr(1, sExpandedSz, vbNullChar, vbBinaryCompare)
            vValue = Left(sExpandedSz, lValueLen - 1)
        End If
        
    Case REG_BINARY '// assume vValue passed as byte array
        ReDim vValue(lValueLen - 1)
        For iIndex = 0 To (lValueLen - 1)
            vValue(iIndex) = aValue(iIndex) '// set bytes arrays equal
        Next '// iIndex
        
    Case REG_DWORD
        vValue = 0&
        
        If (aValue(lValueLen - 1) And &H80) Then '// To work around lack of unsigned long data type
            vValue = &H80000000
            iStartByte = lValueLen - 2
            iBit = 23
        Else
            iStartByte = lValueLen - 1
            iBit = 31
        End If
        
        For iByte = iStartByte To 0 Step -1
            For iByteBit = 7 To 0 Step -1
                If ((aValue(iByte) - (2 ^ iByteBit)) >= 0) Then
                    If (iStartByte = 2) Then '// i.e. creating a negative long
                        vValue = vValue - (2 ^ iBit)
                    Else '// i.e. creating a positive long
                        vValue = vValue + (2 ^ iBit)
                    End If
                    aValue(iByte) = aValue(iByte) - (2 ^ iByteBit)
                End If
                iBit = iBit - 1
            Next '// iByteBit
        Next '// iByte
    End Select
    
    QueryValue = 0
            
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    QueryValue = 1
End Function

Public Function SetKey(hKey As Long, sSubKey As String, sValue As String, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValue = "") Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        SetKey = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        lValueLen As Long, aValue() As Byte, iIndex As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName)  '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            SetKey = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        SetKey = lRet
        Exit Function
    End If
    
    lValueLen = Len(sValue)
    ReDim aValue(lValueLen)
    For iIndex = 0 To (lValueLen - 1)
        aValue(iIndex) = Asc(Mid(sValue, (iIndex + 1), 1))
    Next '// iIndex
    
    aValue(iIndex) = 0 '// add null terminator
    lValueLen = lValueLen + 1 '// add 1 to account for null terminator
    
    lRet = RegSetValueEx(phSubKey, vbNullString, 0&, REG_SZ, aValue(0), lValueLen)
    
    If (lRet) Then
        g_sErrInfo = "Unable to set registry subkey <no name> value."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        SetKey = lRet
        Exit Function
    End If
    
    SetKey = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    SetKey = 1
End Function

Public Function SetValue(hKey As Long, sSubKey As String, sValueName As String, lValueType As Long, vValue As Variant, Optional sMachineName As String) As Long
    On Error GoTo ErrorHandler
    g_sErrInfo = ""
    g_bCancel = False
    
    If (hKey = 0 Or sSubKey = "" Or sValueName = "" Or IsEmpty(lValueType) Or IsEmpty(vValue)) Then
        g_sErrInfo = "A required parameter was passed incorrectly."
        SetValue = 1
        Exit Function
    End If
    
    Dim bRemoteReg As Boolean, phKey As Long, lRet As Long, phSubKey As Long, _
        sValueTemp As String, lValueLen As Long, aValue() As Byte, iIndex As Integer, _
        lValueTemp As Long, iStartByte As Integer, iBit As Integer, _
        iByte As Integer, iByteBit As Integer
        
    If (sMachineName = "") Then
        bRemoteReg = False
        phKey = hKey
    Else
        bRemoteReg = True
        lRet = EnsureUncName(sMachineName) '// ensure sMachineName has UNC name structure
        lRet = RegConnectRegistry(sMachineName, hKey, phKey) '// get phKey
        If (lRet) Then
            g_sErrInfo = "Unable to connect to remote registry."
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
            SetValue = lRet
            Exit Function
        End If
    End If
   
    lRet = RegOpenKeyEx(phKey, sSubKey, 0, KEY_ALL_ACCESS, phSubKey) '// get phSubKey
    
    If (lRet) Then
        If (lRet = ERROR_FILE_NOT_FOUND) Then
            g_sErrInfo = "Unable to connect to registry subkey, ERROR_FILE_NOT_FOUND."
        Else
            g_sErrInfo = "Unable to connect to registry subkey."
        End If
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        SetValue = lRet
        Exit Function
    End If
    
    Select Case (lValueType)
    Case REG_SZ, REG_EXPAND_SZ, REG_MULTI_SZ '// assume vValue passed as string
        sValueTemp = CStr(vValue) '// ensure vValue is cast as a string
        lValueLen = Len(sValueTemp)
        
        If (lValueType = REG_MULTI_SZ) Then
            If (InStr((lValueLen), sValueTemp, vbNullChar, vbBinaryCompare) = 0) Then
                sValueTemp = sValueTemp & vbNullChar
                lValueLen = lValueLen + 1
            End If
        End If
        
        ReDim aValue(lValueLen)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = Asc(Mid(sValueTemp, (iIndex + 1), 1))
        Next '// iIndex
        
        aValue(iIndex) = 0 '// add null terminator
        lValueLen = lValueLen + 1 '// add 1 for null terminator
        
    Case REG_BINARY '// assume vValue passed as byte array
        lValueLen = UBound(vValue) + 1
        ReDim aValue(lValueLen)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = vValue(iIndex) '// set bytes arrays equal
        Next '// iIndex
        
        aValue(iIndex) = 0 '// set null terminator
        
    Case REG_DWORD
        lValueTemp = CLng(vValue)
        lValueLen = MAX_REG_DWORD_LEN
        ReDim aValue(lValueLen - 1)
        For iIndex = 0 To (lValueLen - 1)
            aValue(iIndex) = 0 '// set bytes to 0
        Next '// iIndex
        
        If (lValueTemp And &H80000000) Then '// To work around lack of unsigned long data type
            aValue(lValueLen - 1) = &H80
            iStartByte = lValueLen - 2
            iBit = 23
            lValueTemp = lValueTemp - &H80000000 '// adjust value for calculation of remaining bytes
        Else
            iStartByte = lValueLen - 1
            iBit = 31
        End If
        
        For iByte = iStartByte To 0 Step -1
            For iByteBit = 7 To 0 Step -1
                If ((lValueTemp - (2 ^ iBit)) >= 0) Then
                    aValue(iByte) = aValue(iByte) + (2 ^ iByteBit)
                    lValueTemp = lValueTemp - (2 ^ iBit)
                End If
                iBit = iBit - 1
            Next '// iByteBit
        Next '// iByte
    End Select
        
    lRet = RegSetValueEx(phSubKey, sValueName, 0&, lValueType, aValue(0), lValueLen)
                          
    If (lRet) Then
        g_sErrInfo = "Unable to create registry value."
        Call RegCloseKey(phSubKey)
        Call RegCloseKey(phKey)
        If (bRemoteReg) Then
            Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
        End If
        SetValue = lRet
        Exit Function
    End If
    
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    SetValue = 0
    
    Exit Function
ErrorHandler:
    g_sErrInfo = Err.Number - vbObjectError & ", " & Err.Description & ", " & Err.LastDllError
    Call RegCloseKey(phSubKey)
    Call RegCloseKey(phKey)
    If (bRemoteReg) Then
        Call WNetCancelConnection2(sMachineName & "\ipc$", 0&, False)
    End If
    SetValue = 1
End Function


