// Disk.cpp : Implementation of CDisk
#include "stdafx.h"
#include "IcuUtils.h"
#include "Disk.h"

// additions
#include <aclapi.h>

/////////////////////////////////////////////////////////////////////////////
// CDisk

STDMETHODIMP CDisk::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDisk
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CDisk::GetLargeDriveFree(BSTR sDrivePath, DWORD *pdwTotalFree)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	// This is a new api extension that allows for checking the 
	// available space on storage devices of size >2GB.
	// I haven't tested to see if the Script.FileSystemsObject [IFileSystem]
	// interfaces "getSize()" method is also capable of returning
	// accurate values for these types of large storage devices.
	BOOL bRet = FALSE;
	ULARGE_INTEGER uliFreeToCaller, uliTotal, uliTotalFree;
	DWORD dwFreeToCaller = 0, dwTotal = 0, dwTotalFree = 0;
	
	// zero the variables
	uliFreeToCaller.QuadPart = 0;
	uliTotal.QuadPart = 0;
	uliTotalFree.QuadPart = 0;
	
	bRet = GetDiskFreeSpaceEx(sDrivePath, &uliFreeToCaller, 
							  &uliTotal, &uliTotalFree);
	if (bRet)
	{
		// convert results from bytes to megabytes so value will fit in a DWORD
		dwFreeToCaller = (DWORD)uliFreeToCaller.QuadPart/1000000;
		dwTotal = (DWORD)uliTotal.QuadPart/1000000; 
		dwTotalFree = (DWORD)uliTotalFree.QuadPart/1000000;
	}

	// assign values to out parameters
	*pdwTotalFree = dwTotalFree;

	return S_OK;
}

STDMETHODIMP CDisk::AclModify(BSTR sFileOrFolder, BSTR sUserOrGroup, DWORD dwFileMask, DWORD dwFolderMask, DWORD dwAccessMode)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	
	// dwFileMask options   = GENERIC_READ | GENERIC_WRITE | GENEREIC_EXECUTE | DELETE | GENERIC_ALL
	// dwFolderMask options = FILE_GENERIC_READ | FILE_GENERIC_WRITE | FILE_GENEREIC_EXECUTE |
	//  				      DELETE | FILE_ALL_ACCESS

	// using low level acl api's

	BOOL br = FALSE;
	// SECURITY_DESCRIPTOR sd;

	// SID variables
	_TCHAR psnuType[2048] = {_T("\0")};
	_TCHAR lpszDomain[2048] = {_T("\0")};
	DWORD dwDomainLength = 250;
	_TCHAR UserSID[1024] = {_T("\0")};
	DWORD dwSIDBufSize = 1024;
	int	sidlen = 0;

	// user name variables
	DWORD dwUserNameLength = 250;

	// file SD variables
	#define SD_SIZE  (65536 + SECURITY_DESCRIPTOR_MIN_LENGTH)
	_TCHAR ucSDbuf[SD_SIZE] = {_T("\0")};
	PSECURITY_DESCRIPTOR pFileSD=(PSECURITY_DESCRIPTOR)ucSDbuf;
	DWORD dwSDLengthNeeded = 0;

	// ACL variables
	ACCESS_ALLOWED_ACE *paaa = (ACCESS_ALLOWED_ACE *) malloc (1024);
	PACL pACL = NULL;
	BOOL bDaclPresent = FALSE;
	BOOL bDaclDefaulted = FALSE;
	ACL_SIZE_INFORMATION AclInfo;

	// new ACL variables
	PACL pNewACL = NULL;
	DWORD dwNewACLSize = 0;

	// new SD variables
	UCHAR NewSD[SECURITY_DESCRIPTOR_MIN_LENGTH];
	PSECURITY_DESCRIPTOR psdNewSD=(PSECURITY_DESCRIPTOR)NewSD;

	// temporary ACE
	PVOID pTempAce = NULL;
	UINT CurrentAceIndex = 0;

	try
	{
		// get SID for the user
		br = LookupAccountName(NULL, sUserOrGroup, UserSID, &dwSIDBufSize,
							   lpszDomain, &dwDomainLength, (PSID_NAME_USE)psnuType);
		if (!br) throw (br);
		
		// get security descriptor (SD) for file
		br = GetFileSecurity(sFileOrFolder, (SECURITY_INFORMATION)(DACL_SECURITY_INFORMATION),
							 pFileSD, SD_SIZE, (LPDWORD)&dwSDLengthNeeded);
		if (!br) throw (br);

		// initialize new SD
		br = InitializeSecurityDescriptor(psdNewSD,SECURITY_DESCRIPTOR_REVISION);
		if (!br) throw (br);

		// get DACL from SD
		br = GetSecurityDescriptorDacl(pFileSD,	&bDaclPresent, &pACL, &bDaclDefaulted);
		if (!br) throw (br);

		// get file ACL size information
		br = GetAclInformation(pACL, &AclInfo, sizeof(ACL_SIZE_INFORMATION), AclSizeInformation);
		if (!br) throw (br);

		// compute size needed for the new ACL  (make room for 2 additional ace's
		// one for the container inheritance and one for the file inheritance.
		dwNewACLSize = AclInfo.AclBytesInUse + 2 * (sizeof(ACCESS_ALLOWED_ACE) + 
					   GetLengthSid(UserSID) -  sizeof(DWORD));
		sidlen = GetLengthSid(UserSID);

		// allocate memory for new ACL
		pNewACL = (PACL)LocalAlloc(LPTR, dwNewACLSize);

		// initialize the new ACL
		br = InitializeAcl(pNewACL, dwNewACLSize, ACL_REVISION);
		if (!br) throw (br);

		// if DACL is present, copy it to a new DACL
		if(bDaclPresent)  // only copy if DACL was present
		{
			// copy the file's ACEs to our new ACL
			if(AclInfo.AceCount)
			{
				for(CurrentAceIndex = 0; CurrentAceIndex < AclInfo.AceCount;
					CurrentAceIndex++)
				{
					// get an ACE
					br = GetAce(pACL,CurrentAceIndex,&pTempAce);
					if (!br) throw (br);

					// add the ACE to the new ACL
					br = AddAce(pNewACL, ACL_REVISION, MAXDWORD, pTempAce, ((PACE_HEADER)pTempAce)->AceSize);
					if (!br) throw (br);
				}
			}
		}

		// fill in the first new ACE  
		memcpy(&paaa->SidStart, UserSID, GetLengthSid(UserSID)); 
		paaa->Mask = dwFolderMask; // before = dwFileMask;
		paaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE; 
		paaa->Header.AceFlags = INHERIT_ONLY_ACE | OBJECT_INHERIT_ACE | CONTAINER_INHERIT_ACE;  // before = INHERIT_ONLY_ACE | OBJECT_INHERIT_ACE;
		paaa->Header.AceSize =  sizeof(ACCESS_ALLOWED_ACE) + sidlen - sizeof(DWORD);

		br = AddAce(pNewACL, ACL_REVISION, MAXDWORD, paaa, paaa->Header.AceSize);
		if (!br) throw (br);
		
		/*
		paaa->Mask = dwFolderMask;  // before = dwFileMask;
		paaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE; 
		paaa->Header.AceFlags = 0;  // before = CONTAINER_INHERIT_ACE; 
		paaa->Header.AceSize =  sizeof(ACCESS_ALLOWED_ACE) + sidlen - sizeof(DWORD);

		// before br = AddAce(pNewACL, ACL_REVISION, MAXDWORD, paaa, paaa->Header.AceSize);
		*/
		br = AddAccessAllowedAce(pNewACL, ACL_REVISION, dwFolderMask, UserSID);
		if (!br) throw (br);

		// set our new DACL to the file SD
		br = SetSecurityDescriptorDacl(psdNewSD, TRUE, pNewACL, FALSE);
		if (!br) throw (br);

		// set the SD to the File
		br = SetFileSecurity(sFileOrFolder, DACL_SECURITY_INFORMATION,psdNewSD);
		if (!br) throw (br);

		// free the memory allocated for the new ACL before departing
		LocalFree((HLOCAL)pNewACL);
		return S_OK;
	}

	catch(BOOL brLastError)
    {
		brLastError = FALSE; // for no reason other than to get rid of compiler warning
		// brLastError isn't really the LastError information so send the 0x0 value as the
		// argument for GetLastErrorData() call so that GetLastError() within that 
		// method will be used to derive the actual LastError Number and Description
		// GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc
		
		// free the memory allocated for the new ACL before departing
		if (pNewACL)
            LocalFree((HLOCAL)pNewACL);

		return E_FAIL;
    }

	catch(...)  // any type including C, system and application generated exceptions
    {
		// GetLastErrorData(0x0);  // sets m_lLastErrorNum and m_sLastErrorDesc

		// free the memory allocated for the new ACL before departing
		if (pNewACL)
            LocalFree((HLOCAL)pNewACL);

		return E_FAIL;
	}

	return S_OK;
}
