// ShortCut.cpp : Implementation of CShortCut
#include "stdafx.h"
#include "IcuUtils.h"
#include "ShortCut.h"

/////////////////////////////////////////////////////////////////////////////
// CShortCut

STDMETHODIMP CShortCut::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IShortCut
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CShortCut::ParseCmdLine()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	HRESULT hr = E_FAIL;
	m_bFoundCmdLineArg = FALSE;  // initialize to FALSE
	m_bFoundHelpArg = FALSE;  // initialize to FALSE

	CString sCmdLine = m_sCmdLine;
	if (0 == sCmdLine.GetLength())
	{
		m_bFoundCmdLineArg = FALSE;
		hr = E_FAIL;
		return hr;
	}
	m_sCmdLine = sCmdLine;
	TCHAR szArgSepChar[] = _T("/");  // _T("-/");
	// Removed '-' character from token separator list
	// since this ended up in arguments on a regular
	// basis when this routine was called by shortcut.exe.
	// Removing support for the '-' token was easier
	// than building a safe array that could be used to
	// check against when trying to decide if instances
	// of '-' were being used as token delimeters or as
	// part of an argument value.
	if (-1 == sCmdLine.FindOneOf(szArgSepChar))
	{
		m_bFoundCmdLineArg = FALSE;
		return S_OK;
	}
	sCmdLine = sCmdLine.Mid(sCmdLine.FindOneOf(szArgSepChar) + 1);
	int nNextToken = 0;
	while (0 != sCmdLine.GetLength())
	{
		// 1 == sCmdLine.GetLength() catches cases where just "/h" | "/?" or "/<arg specifier but no value>" was provided
		if (1 == sCmdLine.GetLength() || 0 == sCmdLine.Left(2).CompareNoCase(_T("h ")) || 0 == sCmdLine.Left(2).CompareNoCase(_T("? ")))
		{
			m_bFoundCmdLineArg = TRUE;
			m_bFoundHelpArg = TRUE;
			sCmdLine.Empty();  // empty so that we don't process any additional arguments
			
		}
		if (0 == sCmdLine.Left(2).CompareNoCase(_T("l ")))
		{
			m_bFoundCmdLineArg = TRUE;
			nNextToken = sCmdLine.FindOneOf(szArgSepChar);
			if (0 < nNextToken)
				m_sLink = sCmdLine.Mid(2, (nNextToken - 2));
			else
			{
				m_sLink = sCmdLine.Mid(2);
				sCmdLine.Empty();
			}
		}

		if (0 == sCmdLine.Left(2).CompareNoCase(_T("p ")))
		{
			m_bFoundCmdLineArg = TRUE;
			nNextToken = sCmdLine.FindOneOf(szArgSepChar);
			if (0 < nNextToken)
				m_sPath = sCmdLine.Mid(2, (nNextToken - 2));
			else
			{
				m_sPath = sCmdLine.Mid(2);
				sCmdLine.Empty();
			}
		}

		if (0 == sCmdLine.Left(2).CompareNoCase(_T("a ")))
		{
			m_bFoundCmdLineArg = TRUE;
			nNextToken = sCmdLine.FindOneOf(szArgSepChar);
			if (0 < nNextToken)
				m_sArguments = sCmdLine.Mid(2, (nNextToken - 2));
			else
			{
				m_sArguments = sCmdLine.Mid(2, (nNextToken - 2));
				sCmdLine.Empty();
			}
		}

		if (0 == sCmdLine.Left(2).CompareNoCase(_T("w ")))
		{
			m_bFoundCmdLineArg = TRUE;
			nNextToken = sCmdLine.FindOneOf(szArgSepChar);
			if (0 < nNextToken)
				m_sWorkingDir = sCmdLine.Mid(2, (nNextToken - 2));
			else
			{
				m_sWorkingDir = sCmdLine.Mid(2);
				sCmdLine.Empty();
			}
		}

		if (0 == sCmdLine.Left(2).CompareNoCase(_T("d ")))
		{
			m_bFoundCmdLineArg = TRUE;
			nNextToken = sCmdLine.FindOneOf(szArgSepChar);
			if (0 < nNextToken)
				m_sDescription = sCmdLine.Mid(2, (nNextToken - 2));
			else
			{
				m_sDescription = sCmdLine.Mid(2);
				sCmdLine.Empty();
			}
		}
		sCmdLine = sCmdLine.Mid(sCmdLine.FindOneOf(szArgSepChar) + 1);
	}
	return S_OK;
}

STDMETHODIMP CShortCut::CreateOrModify()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	HRESULT hr = E_FAIL;

	// declare IShellLink and IPersistFile interface smart pointers
	CComPtr<IShellLink> pShellLink;
	CComPtr<IPersistFile> pPersistFile;

	// the IShellLink interface was not defined using the "MIDL_INTERFACE" macro
	// in its header file <shlobj.h> therefore we will need to revert to using
	// the CoCreateInstance API that requires more arguments
	// hr = spShellLink.CoCreateInstance(_uuidof(ShellLink), NULL, CLSCTX_ALL);
	hr = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_ALL, IID_IShellLink, (void **)&pShellLink);
	if(FAILED(hr)) return hr;

	// the IPersist interface was defined using the "MIDL_INTERFACE" macro 
	// in its header file <shlobj.h> therefore we can use the CComPtr 
	// QueryInterface method that requires fewer arguments than the API version
	hr = pShellLink.QueryInterface(&pPersistFile);
	if(FAILED(hr)) return hr;

	if (0 != m_sPath.GetLength())
	{
		m_sPath.TrimLeft();  // remove if present leading spaces
		m_sPath.TrimRight();  // remove if present trailing spaces
		m_sPath.Remove('\"');  // remove if present quotation wrappers
		hr = pShellLink->SetPath(m_sPath);
		if(FAILED(hr)) return hr;
	}
	else
	{
		// return failure since this argument is required
		hr = E_FAIL;
		return hr;
	}

	if (0 != m_sArguments.GetLength())
	{
		m_sArguments.TrimLeft();  // remove if present leading spaces
		m_sArguments.TrimRight();  // remove if present trailing spaces
		m_sArguments.Remove('\"');  // remove if present quotation wrappers
		hr = pShellLink->SetArguments(m_sArguments);
		if(FAILED(hr)) return hr;
	}

	if (0 != m_sWorkingDir.GetLength())
	{
		m_sWorkingDir.TrimLeft();  // remove if present leading spaces
		m_sWorkingDir.TrimRight();  // remove if present trailing spaces
		m_sWorkingDir.Remove('\"');  // remove if present quotation wrappers
		hr = pShellLink->SetWorkingDirectory(m_sWorkingDir);
		if (FAILED(hr)) return hr;
	}

	if (0 != m_sDescription.GetLength())
	{
		m_sDescription.TrimLeft();  // remove if present leading spaces
		m_sDescription.TrimRight();  // remove if present trailing spaces
		m_sDescription.Remove('\"');  // remove if present quotation wrappers
		hr = pShellLink->SetDescription(m_sDescription);
		if (FAILED(hr)) return hr;
	}

	if (0 != m_sLink.GetLength())
	{
		m_sLink.TrimLeft();  // remove if present trailing spaces
		m_sLink.TrimRight();  // remove if present trailing spaces
		m_sLink.Remove('\"');  // remove if present quotation wrappers
		// check if ".lnk" extension was provided and if not appended it to link name
		if (0 != m_sLink.Right(4).CompareNoCase(_T(".lnk")))
			m_sLink += _T(".lnk");
		hr = pPersistFile->Save((LPCTSTR)m_sLink, TRUE);
		if(FAILED(hr)) return hr;
	}
	else
	{
		// return failure since this argument is required
		hr = E_FAIL;
		return hr;
	}

	/*
	// interface pointer release method not required as it
	// is handled automatically by use of smart pointers
	pPersistFile->Release();
	pShellLink->Release();
	*/

	return S_OK;
}

STDMETHODIMP CShortCut::get_CmdLine(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sCmdLine.GetBuffer(m_sDescription.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_CmdLine(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sCmdLine = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_FoundCmdLineArg(BOOL *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_bFoundCmdLineArg;

	return S_OK;
}

STDMETHODIMP CShortCut::put_FoundCmdLineArg(BOOL newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_bFoundCmdLineArg = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_FoundHelpArg(BOOL *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_bFoundHelpArg;

	return S_OK;
}

STDMETHODIMP CShortCut::put_FoundHelpArg(BOOL newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_bFoundHelpArg = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_Link(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sLink.GetBuffer(m_sLink.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_Link(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sLink = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_Path(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sPath.GetBuffer(m_sPath.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_Path(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sPath = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_Arguments(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sArguments.GetBuffer(m_sArguments.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_Arguments(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sArguments = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_WorkingDir(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sWorkingDir.GetBuffer(m_sWorkingDir.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_WorkingDir(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sWorkingDir = newVal;

	return S_OK;
}

STDMETHODIMP CShortCut::get_Description(BSTR *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	*pVal = m_sDescription.GetBuffer(m_sDescription.GetLength());

	return S_OK;
}

STDMETHODIMP CShortCut::put_Description(BSTR newVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_sDescription = newVal;

	return S_OK;
}
