// Adapters.cpp: implementation of the CAdapters class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ifConfig.h"
#include "Adapters.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAdapters::CAdapters()
{

}


CAdapters::~CAdapters()
{

}


HRESULT CAdapters::DisplayFormatMessage(DWORD dwMessageId)
{
	// check if caller passed GET_LAST_ERROR == dwMessageId 
	// and if so acquire the dwMessageId from GetLastError()
	if (GET_LAST_ERROR == dwMessageId)  
		dwMessageId = GetLastError();  

	DWORD dwFormatFlags = (FORMAT_MESSAGE_ALLOCATE_BUFFER |
						   FORMAT_MESSAGE_IGNORE_INSERTS |
						   FORMAT_MESSAGE_FROM_SYSTEM);

	HMODULE hModule = NULL;
    // using hModule = NULL defaults to the standard system dlls 
	// for error id to message string lookups. if dwMessageId is 
	// in the network error id range then set hModule to refer to
	// the dll containing the network error id message strings.
    if ((NERR_BASE <= dwMessageId) && (MAX_NERR >= dwMessageId)) 
	{
        hModule = LoadLibraryEx(_T("netmsg.dll"), NULL, LOAD_LIBRARY_AS_DATAFILE);
        if (hModule)
			dwFormatFlags |= FORMAT_MESSAGE_FROM_HMODULE;
    }

    // use FormatMessage() to acquire the message string for 
	// dwMessageId using the default system language id
	LPTSTR suMessageStr = NULL;
	UINT cbMessageStr = FormatMessage(dwFormatFlags, hModule, dwMessageId,
								 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
								 (LPTSTR)&suMessageStr, 0, NULL);
    if (cbMessageStr)
    {
		TCHAR suCaption[MAX_CB_BUFFER] = {_T("DisplayFormatMessage")},
			  suFormatMessage[MAX_CB_BUFFER] = {_T("\0")};

		wsprintf(suFormatMessage, _T("A system call returned message id =  %lu with message string = \n%s."), dwMessageId, suMessageStr);
		
		/*
		#ifdef _DEBUG  // if we are in debug mode optionally output to stderr
			DWORD dwBytesWritten = 0;
			WriteFile(GetStdHandle(STD_ERROR_HANDLE), suFormatMessage, 
					  (sizeof(suFormatMessage), &dwBytesWritten, NULL);
		#endif
		*/

		// if this is a console application then output the format message to the console
		wcout << suFormatMessage << endl;
		wcout << _T(" ") << endl;
		// OR
		// wprintf(%s\n, suFormatMessage);

		// if this is a windows application then output the format message to a MessageBox
		// INT nr = MessageBox(NULL, suFormatMessage, suCaption,  MB_OK | MB_ICONINFORMATION);
       
        // if this is a COM interface assign format message values to member variables
		// _tcscpy(m_suMessageStr, suFormatMessage);
		        
		// free the buffer allocated by the format message call
		LocalFree(suMessageStr);
    }

    // if we loaded a message source, unload it
    if (hModule)
		FreeLibrary(hModule);

	return S_OK;
}


HRESULT CAdapters::IpStringToHex(LPCTSTR psuIpAddr, DWORD *pdwIpAddr)
{
	char szIpAddr[MAX_CB_IPADDRESS] = {"\0"};
	INT nr = wcstombs(szIpAddr, psuIpAddr, sizeof(TCHAR) * MAX_CB_IPADDRESS);	
	IPAddr tIpAddr;
	// tIpAddr.s_addr = 0;
	tIpAddr = inet_addr(szIpAddr);
	*pdwIpAddr = ntohl(tIpAddr);

	return S_OK;
}


HRESULT CAdapters::IpHexToString(DWORD dwIpAddr, LPTSTR psuIpAddr)
{
	IN_ADDR tIpAddr;
	tIpAddr.s_addr = 0;
	tIpAddr.s_addr = htonl(dwIpAddr);  // assigns network hex representation in structure
	char *szIpAddr = NULL;
	szIpAddr = inet_ntoa(tIpAddr);  
	INT nr = mbstowcs(psuIpAddr, szIpAddr, MAX_CB_IPADDRESS);	

	return S_OK;
}


HRESULT CAdapters::List(LPCTSTR psuIfArg)
{
	try
	{
		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if (ERROR_NO_DATA != dwr)
		{
			if ((NO_ERROR != dwr) &&
				(ERROR_BUFFER_OVERFLOW != dwr))
				throw (dwr);

			// allocate memory from sizing information
			PIP_ADAPTER_INFO ptIfsInfo = NULL;
			ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
			if (!ptIfsInfo) throw (GET_LAST_ERROR);

			// get actual adapter information
			dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
			if (dwr) throw (dwr);

			// setup and initialize variables used within loop
			DWORD dwr = 0, dwInet = 0, dwNetmask = 0, dwBroadcast = 0,
				  dwSpeed = 0, dwDhcpMac = 0;
			HRESULT hr = E_FAIL;
			INT nr = 0;
			MIB_IFROW tIfRow;
			PIP_ADDR_STRING ptAddrStr = NULL;
			TCHAR suIf[] = {_T("***")},  // "if0", "if1", etc.
				  suBuffer[MAX_ADAPTER_DESCRIPTION_LENGTH + 4] = {_T("\0")},
				  suDesc[MAX_ADAPTER_DESCRIPTION_LENGTH + 4] = {_T("\0")},
				  suUpStatus[] = {_T("****")},  // "UP" | "DOWN"
				  suAddrType[] = {_T("******")},  // "DHCP" | "STATIC"
				  suInet[MAX_CB_IPADDRESS] = {_T("***.***.***.***")},
				  suNetmask[MAX_CB_IPADDRESS] = {_T("***.***.***.***")},
				  suBroadcast[MAX_CB_IPADDRESS] = {_T("***.***.***.***")},
				  suSpdUnits[] = {_T("****")},
				  suDhcpMac[] = {_T("********")};
			TCHAR *psuSubStr = NULL;

			while (ptIfsInfo)
			{
				// use GetFriendlyIfIndex(ptIfsInfo->Index) value which is 
				// non-zero based but always unique
				wsprintf(suIf, _T("if%d"), GetFriendlyIfIndex(ptIfsInfo->Index));
				// OR use ptIfsInfo->ComboIndex value which is zero based but
				// doesn't appear to be always unique for each adapter in all cases
				// wsprintf(suIf, _T("if%d"), ptIfsInfo->ComboIndex);

				tIfRow.dwIndex = ptIfsInfo->Index;
				dwr = GetIfEntry(&tIfRow);
				if (NO_ERROR != dwr) throw (dwr);

				ptAddrStr = &(ptIfsInfo->IpAddressList);
				nr = mbstowcs(suInet, ptAddrStr->IpAddress.String, MAX_CB_IPADDRESS);
				dwInet = 0;
				hr = IpStringToHex(suInet, &dwInet);
				if (!_tcscmp(suIf, psuIfArg) ||
					!_tcscmp(_T("all"), psuIfArg) ||
				   (!_tcscmp(_T("up"), psuIfArg) && (dwInet)) ||
				   (!_tcscmp(_T("down"), psuIfArg) && !(dwInet)))
				{
					if (dwInet)
						_tcscpy(suUpStatus, _T("up"));
					else
						_tcscpy(suUpStatus, _T("down"));

					if ((MIB_IF_TYPE_PPP == ptIfsInfo->Type)  ||
						(MIB_IF_TYPE_SLIP == ptIfsInfo->Type))
						_tcscpy(suAddrType, _T("dialup"));
					else  // (MIB_IF_TYPE_ETHERNET || _TOKENRING || _FDDI || _LOOPBACK || _OTHER
						(ptIfsInfo->DhcpEnabled) ? _tcscpy(suAddrType, _T("dhcp")) : _tcscpy(suAddrType, _T("static"));

					_tcsnset(suBuffer, '\0', (MAX_ADAPTER_DESCRIPTION_LENGTH + 4));
					_tcsnset(suDesc, '\0', (MAX_ADAPTER_DESCRIPTION_LENGTH + 4));
					nr = mbstowcs(suBuffer, ptIfsInfo->Description, (MAX_ADAPTER_DESCRIPTION_LENGTH + 4));
					psuSubStr = NULL;
					psuSubStr = _tcsstr(suBuffer, _T("(Mi")); // looking for "(Microsoft's Packet Scheduler) "
					if (psuSubStr)
						_tcsncpy(suDesc, suBuffer, (psuSubStr - suBuffer - 1));
					else
						_tcscpy(suDesc, suBuffer);

					// output format desired in order to match existing solution
					//	"le0: flags=863<UP,BROADCAST,NOTRAILERS,RUNNING,MULTICAST> mtu 1500"
					dwSpeed = tIfRow.dwSpeed;
					// _tcsset(suSpdUnits, '\0');
					if (dwSpeed > pow(10, 6))
					{
						dwSpeed /= pow(10, 6);
						_tcscpy(suSpdUnits, _T("Mbps"));
					}
					else
					{
						dwSpeed /= pow(10, 3);
						_tcscpy(suSpdUnits, _T("Kbps"));
					}

					DWORD dwMtu = tIfRow.dwMtu;
					// wprintf(_T("\n%s: spd=%u<%s,%s,%s> mtu=%u\n"), suIf, dwSpeed, suUpStatus, suAddrType, suDesc, dwMtu);
					wprintf(_T("\n%s: spd=%u%s<%s,%s,%s> mtu %u\n"), suIf, dwSpeed, suSpdUnits, suUpStatus, suAddrType, suDesc, dwMtu);
					
					while(ptAddrStr)
					{
						// output format desired in order to match existing solution
						// "	inet 157.54.156.37 netmask fffffc00 broadcast 157.54.159.255"
						dwInet = 0;
						nr = mbstowcs(suInet, ptAddrStr->IpAddress.String, MAX_CB_IPADDRESS);
						hr = IpStringToHex(suInet, &dwInet);
					
						dwNetmask = 0;
						nr = mbstowcs(suNetmask, ptAddrStr->IpMask.String, MAX_CB_IPADDRESS);
						hr = IpStringToHex(suNetmask, &dwNetmask);

						dwBroadcast = (dwInet & dwNetmask);
						hr = IpHexToString(dwBroadcast, suBroadcast);
						// wprintf(_T("\tinet %s netmask %s broadcast %s\n"), suInet, suNetmask, suBroadcast);
						// OR
						wprintf(_T("\tinet %s netmask %.8x broadcast %s\n"), suInet, dwNetmask, suBroadcast);
						ptAddrStr = ptAddrStr->Next;
					}
					
					ptAddrStr = &(ptIfsInfo->GatewayList);
					while(ptAddrStr)
					{
						// output format desired no existing solution to match ???
						// "	gateway 157.54.156.37 netmask fffffc00"
						
						nr = mbstowcs(suInet, ptAddrStr->IpAddress.String, MAX_CB_IPADDRESS);
						if (!_tcslen(suInet))
						{
							_tcscpy(suInet, _T("0.0.0.0"));
							dwInet = 0;
						}
						else
							hr = IpStringToHex(suInet, &dwInet);
						
						nr = mbstowcs(suNetmask, ptAddrStr->IpMask.String, MAX_CB_IPADDRESS);
						if (!_tcslen(suNetmask) || !_tcscmp(_T("0.0.0.0"), suNetmask))  
						{
							if (!dwInet)  // and dwInet = 0 also
							{
								_tcscpy(suNetmask, _T("0.0.0.0"));
								dwNetmask = 0;
							}
							else
							{
								// then netmask is just the same as the one used by
								// the inet addresses thus use that value versus
								// a value of 0x0000000 since it will confuse user
								hr = IpHexToString(dwNetmask, suNetmask);
							}
						}
						else
							hr = IpStringToHex(suNetmask, &dwNetmask);
							

						dwBroadcast = (dwInet & dwNetmask);
						hr = IpHexToString(dwBroadcast, suBroadcast);
						// wprintf(_T("\tgateway %s\n"), suInet);
						// OR gateway and netmask values
						wprintf(_T("\tgateway %s netmask %.8x\n"), suInet, dwNetmask);
						// OR gateway, netmask and broadcast address values
						// wprintf(_T("\tgateway %s netmask %.8x broadcast %s\n"), suInet, dwNetmask, suBroadcast);
						ptAddrStr = ptAddrStr->Next;
					}

					// output format desired in order to match existing solution
					// "	ether 8:0:20:72:ec:0"
					wprintf(_T("\tether "));
					for (UINT i = 0; i < ptIfsInfo->AddressLength; i++)
					{
						if (i == (ptIfsInfo->AddressLength - 1))
							wprintf(_T("%.2X\n"), (int)ptIfsInfo->Address[i]);
						else
							wprintf(_T("%.2X:"), (int)ptIfsInfo->Address[i]);
					}
					// look up and if present output the dhcpMac address
					// (aka DhcpClientIdentifier) configured in the registry
					// to override NIC burned in mac address thus defining
					// what ip address this host will be issued via dhcp
					if (ptIfsInfo->DhcpEnabled)
					{
						hr = GetIfDhcpMac(ptIfsInfo->Index, &dwDhcpMac);
						if (dwDhcpMac)  // if dhcpMac is set (i.e. > 0) then output value
						{
							_ultow(dwDhcpMac, suDhcpMac, 16);
							wprintf(_T("\tdhcpMac "));
							for (UINT i = 0; i < _tcslen(suDhcpMac); i++)
							{
								if (i == (_tcslen(suDhcpMac) - 1))
									wprintf(_T("%.2X\n"), (int)suDhcpMac[i]);
								else
									wprintf(_T("%.2X:"), (int)suDhcpMac[i]);
							}
						}
					}
				}  
				ptIfsInfo = ptIfsInfo->Next;
			}
		}	
		else  // no interface data
		{
			wprintf(_T("\nNo data is currently available on interface \"%s\".\n"), psuIfArg);
			wprintf(_T("This may be because the interface is currently disconnected.\n"));
			wprintf(_T("Logically or Physically connect the interface and then try again..\n"));
		}

		return S_OK;
	}
	
	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}

}


HRESULT CAdapters::ChgIfStatus(LPCTSTR psuIf, BOOL bUp)
{  
	try
	{
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);
		if (S_OK != hr) throw (hr);
		
		MIB_IFROW tIfRow;
		tIfRow.dwIndex = dwIfIndex;

		DWORD dwr = GetIfEntry(&tIfRow);
		if (NO_ERROR != dwr) throw (dwr);

		if (bUp)
			tIfRow.dwAdminStatus = MIB_IF_ADMIN_STATUS_UP;
		else
			tIfRow.dwAdminStatus = MIB_IF_ADMIN_STATUS_DOWN;

		dwr = SetIfEntry(&tIfRow);
		if (NO_ERROR != dwr) throw (dwr);

		return S_OK;
	}
	
	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfDhcpStatus(LPCTSTR psuIf, BOOL bRenew)
{
	try
	{
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);
		if (S_OK != hr) throw (hr);

		// get sizing information about all interfaces
		DWORD dwIfsIfInfoSize = 0;
		DWORD dwr = GetInterfaceInfo(NULL, &dwIfsIfInfoSize);
		if ((NO_ERROR != dwr) && (ERROR_INSUFFICIENT_BUFFER != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_INTERFACE_INFO ptIfsIfInfo = (PIP_INTERFACE_INFO)GlobalAlloc(GPTR, dwIfsIfInfoSize);
		if (!ptIfsIfInfo) throw (GET_LAST_ERROR);

		// get actual interface information
		dwr = GetInterfaceInfo(ptIfsIfInfo, &dwIfsIfInfoSize);
		if (NO_ERROR != dwr) throw (dwr);

		PIP_ADAPTER_INDEX_MAP ptIfIdxMap = NULL;
		for (INT nIndex = 0; nIndex < ptIfsIfInfo->NumAdapters; nIndex++)
		{
			if (dwIfIndex == ptIfsIfInfo->Adapter[nIndex].Index)
			{
				ptIfIdxMap = &ptIfsIfInfo->Adapter[nIndex];
				break;  // found interfaces interface info break from loop
			}
		}
	
		if (!ptIfIdxMap) return S_FALSE;
		
		if (bRenew)
			dwr = IpRenewAddress(ptIfIdxMap);
		else
			dwr = IpReleaseAddress(ptIfIdxMap);
		
		// if (NO_ERROR != dwr) throw (dwr);
		if (NO_ERROR != dwr) return S_FALSE;

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfDhcpAddr(LPCTSTR psuIf, DWORD dwDhcpEnabled, BOOL bPersist)
{
	try
	{
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);
		if (S_OK != hr) throw (hr);

		DWORD dwCurrentDhcpMode = 0;
		hr = GetIfDhcpMode(dwIfIndex, &dwCurrentDhcpMode);
		if (S_OK != hr) throw (hr);
		if ((dwDhcpEnabled & !dwCurrentDhcpMode) ||
			(!dwDhcpEnabled & dwCurrentDhcpMode))
		{
			wcout << _T(" ") << endl;
			wcout << _T("note - when switching between dhcp and static addressing a reboot") << endl;
			wcout << _T("is required for settings to become active. Use the system provided") << endl;
			wcout << _T("netsh.exe utility if you need to avoid requiring a reboot when") << endl;
			wcout << _T("attempting to carry out this type of configuration change.") << endl;
			// todo - add an option for user to break out of changes
			// after reviewing this note and use netsh.exe instead
		}

		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((NO_ERROR != dwr) && (ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);
  
		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);

	    // get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (NO_ERROR != dwr) throw (dwr);
   
		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo && !ptIfInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info break from loop
			}

			ptIfsInfo = ptIfsInfo->Next;
		}

		// change in memory setting
		/*
		if (!dwDhcpEnabled) // release dhcp address before turning off dhcp addressing
		{	
		    // todo - need to resolve way to detect if current dhcp address
			// was autoAssigned in which case we want to avoid calling a
			// dhcp address release as it takes a long time to time out
			hr = ChgIfDhcpStatus(psuIf, FALSE);  
			if (S_OK != hr) throw (hr);
		}
		*/
		// note - no published iphlpapi currently exists 
		// to allow changing the in memory ip addressing
		// for an interface from dhcp to static of visa versa.
		// When it does we will place the call here for example,
		// dwr = SetDhcpMode(ptIfInfo, dwDhcpEnabled); 
		// if (NO_ERROR != dwr) throw (dwr);

		if (bPersist)  // persist dhcp address changes
		{
			TCHAR suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
			TCHAR suIfName[MAX_ADAPTER_NAME_LENGTH] = {_T("\0")};
			hr = GetIfName(dwIfIndex, suIfName);
			_ultow(dwDhcpEnabled, suIfIpRegValue, 16);
			hr = SetIfIpRegValue(suIfName, _T("EnableDHCP"), REG_DWORD, suIfIpRegValue);
			if (dwDhcpEnabled)  // EnableDHCP == 0x1 default settings
			{
				_tcsset(suIfIpRegValue, '\0');
				hr = SetIfIpRegValue(suIfName, _T("DefaultGateway"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				_tcscpy(suIfIpRegValue, _T("0.0.0.0"));
				hr = SetIfIpRegValue(suIfName, _T("IPAddress"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				hr = SetIfIpRegValue(suIfName, _T("SubnetMask"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
			}
			//else  // EnableDHCP == 0x0 default settings
			//{
				// pretty sure nothing is necessary for ths case
			//}
		}
		return S_OK;
	}
	
	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfDhcpMac(LPCTSTR psuIf, DWORD dwDhcpMac)
{
	// a dwDhcpMac = 0x00000000 value implies use of the physical
	// NIC's MAC address, no bPersist flag argument is required since
	// persistence of this setting is implied.
	try
	{
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);
		if (S_OK != hr) throw (hr);

		/*  since we don't know if they also asked for
			dhcp to be turned on during this pass then
			we will skip this output
		DWORD dwCurrentDhcpMode = 0;
		hr = GetIfDhcpMode(dwIfIndex, &dwCurrentDhcpMode);
		if (S_OK != hr) throw (hr);
		if (!dwCurrentDhcpMode)
		{
			wcout << _T(" ") << endl;
			wcout << _T("note - dhcp is not currently enabled, this setting will") << endl;
			wcout << _T("not take effect until dhcp services are enabled") << endl;
		}
		*/

		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((NO_ERROR != dwr) && (ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);
  
		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);

	    // get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (NO_ERROR != dwr) throw (dwr);
   
		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo && !ptIfInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info break from loop
			}

			ptIfsInfo = ptIfsInfo->Next;
		}

		// change in memory setting
		/*
		if (!dwDhcpEnabled) // release dhcp address before turning off dhcp addressing
		{	
		    // todo - need to resolve way to detect if current dhcp address
			// was autoAssigned in which case we want to avoid calling a
			// dhcp address release as it takes a long time to time out
			hr = ChgIfDhcpStatus(psuIf, FALSE);  
			if (S_OK != hr) throw (hr);
		}
		*/
		// note - no published iphlpapi currently exists 
		// to allow changing the in memory ip addressing
		// for an interface from dhcp to static of visa versa.
		// When it does we will place the call here for example,
		// dwr = SetDhcpMode(ptIfInfo, dwDhcpEnabled); 
		// if (NO_ERROR != dwr) throw (dwr);

		// if (bPersist)  // persist dhcpMac address entry is implied
		// {
			TCHAR suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
			TCHAR suIfName[MAX_ADAPTER_NAME_LENGTH] = {_T("\0")};
			hr = GetIfName(dwIfIndex, suIfName);
			DWORD dwDhcpMacInReg = 0;
			dwDhcpMacInReg = htonl(dwDhcpMac);  // registry value is stored in network order
			_ultow(dwDhcpMacInReg, suIfIpRegValue, 16);
			hr = SetIfIpRegValue(suIfName, _T("DhcpClientIdentifier"), REG_DWORD, suIfIpRegValue);
		// }

		return S_OK;
	}
	
	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfIpAddr(LPCTSTR psuIf, LPCTSTR psuInet, LPCTSTR psuNetmask, BOOL bAdd, BOOL bPersist)
{
	try
	{
		char szInet[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		INT nr = wcstombs(szInet, psuInet, sizeof(TCHAR) * MAX_CB_IPADDRESS);	
		IPAddr tInet = inet_addr(szInet);
		if (!tInet || (INADDR_NONE == tInet)) throw (GET_LAST_ERROR);

		char szNetmask[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		nr = wcstombs(szNetmask, psuNetmask, sizeof(TCHAR) * MAX_CB_IPADDRESS);
		IPMask tNetmask = inet_addr(szNetmask);
		if (!tNetmask || (INADDR_NONE == tNetmask)) throw (GET_LAST_ERROR);
     
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);

		// set and initialize variables used in if block
		BOOL bEntryFound = FALSE;
		DWORD dwr = 0, dwDhcpEnabled = 0;
		TCHAR suIfName[MAX_ADAPTER_NAME_LENGTH] = {_T("\0")}, 
			  suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
		if (bAdd)  // add static address or switch to dhcp addressing
		{
			// if interface is current dhcpEnabled then release 
			// the address and disable dhcp addressing before
			// applying a static address
			hr = GetIfDhcpMode(dwIfIndex, &dwDhcpEnabled);
			if (S_OK != hr) throw (hr);
			
			if (dwDhcpEnabled) 
			{
				hr = ChgIfDhcpAddr(psuIf, FALSE, bPersist);  // disable dhcp addressing
				if (S_OK != hr) throw (hr);
			}

			ULONG ulNteContext = 0, ulNteInstance = 0;
			dwr = AddIPAddress(tInet, tNetmask, dwIfIndex, &ulNteContext, &ulNteInstance);
			// if (NO_ERROR != dwr) throw (dwr)
			// do not throw error in case in mode where user forgot to 
			// to use persist switch when address was initially added
			// and is now running add address switch combination again
			// with the persist switch added, just display AddIPAddress
			// error message and continue.
			if (NO_ERROR != dwr)
			{
				hr = DisplayFormatMessage(dwr);
				// if (S_OK != hr) throw (hr);
			}
		
			if (bPersist)  // persist add static address changes
			{
				_tcsset(suIfIpRegValue, '\0');
				hr = GetIfName(dwIfIndex, suIfName);
				if (S_OK != hr) throw (hr);
				hr = GetIfIpRegValue(suIfName, _T("IPAddress"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				if (!_tcscmp(_T("0.0.0.0"), suIfIpRegValue))
					_tcscpy(suIfIpRegValue, psuInet);
				else
					hr = ChgIfIpMultiValue(suIfIpRegValue, psuInet, TRUE, FALSE, &bEntryFound);
				if (S_OK != hr) throw (hr);
				if (!bEntryFound)
				{
					hr = SetIfIpRegValue(suIfName, _T("IPAddress"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
					hr = GetIfIpRegValue(suIfName, _T("SubnetMask"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
					if (!_tcscmp(_T("0.0.0.0"), suIfIpRegValue))
						_tcscpy(suIfIpRegValue, psuNetmask);
					else
						hr = ChgIfIpMultiValue(suIfIpRegValue, psuNetmask, TRUE, TRUE, &bEntryFound);
					if (S_OK != hr) throw (hr);
					hr = SetIfIpRegValue(suIfName, _T("SubnetMask"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
				}
			}
		}

		else  // delete address
		{
			DWORD dwIpAddrContext = 0;
			hr = GetIpAddrContext(dwIfIndex, psuInet, psuNetmask, &dwIpAddrContext);
			// if (S_OK != hr) throw (hr);

			dwr = DeleteIPAddress(dwIpAddrContext);
			// if (NO_ERROR != dwr) throw (dwr)
			// do not throw error in case in mode where user forgot to 
			// to use persist switch when address was initially deleted
			// and is now running delete address switch combination again
			// with the persist switch added, just display DeleteIPAddress
			// error message and continue.
			if (NO_ERROR != dwr)
			{
				hr = DisplayFormatMessage(dwr);
				// if (S_OK != hr) throw (hr);
			}
		
			if (bPersist)  // persist delete address changes
			{
				_tcsset(suIfIpRegValue, '\0');
				hr = GetIfName(dwIfIndex, suIfName);
				if (S_OK != hr) throw (hr);
				hr = GetIfIpRegValue(suIfName, _T("IPAddress"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				hr = ChgIfIpMultiValue(suIfIpRegValue, psuInet, FALSE, FALSE, &bEntryFound);
				if (S_OK != hr) throw (hr);
				if (bEntryFound)
				{
					hr = SetIfIpRegValue(suIfName, _T("IPAddress"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
					hr = GetIfIpRegValue(suIfName, _T("SubnetMask"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
					hr = ChgIfIpMultiValue(suIfIpRegValue, psuNetmask, FALSE, TRUE, &bEntryFound);
					if (S_OK != hr) throw (hr);
					hr = SetIfIpRegValue(suIfName, _T("SubnetMask"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
				}
			}
		}
		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfGwAddr(LPCTSTR psuIf, LPCTSTR psuInet, LPCTSTR psuNetmask, LPCTSTR psuGw, BOOL bAdd, BOOL bPersist)
{
	try
	{		
		// convert passed arguments to multibyte strings and 
		// generate IPAddr values for use later in routine
		char szInet[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		INT nr = wcstombs(szInet, psuInet, sizeof(TCHAR) * MAX_CB_IPADDRESS);
		IPAddr tInet = inet_addr(szInet);
		
		// set the default gateway destination netmask which is "0.0.0.0"
		char szNetmask[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		// nr = wcstombs(szNetmask, psuNetmask, sizeof(TCHAR) * MAX_CB_IPADDRESS);
		strcpy(szNetmask, "0.0.0.0");
		IPMask tNetmask = inet_addr(szNetmask);

		char szGw[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		nr = wcstombs(szGw, psuGw, sizeof(TCHAR) * MAX_CB_IPADDRESS);	
		IPAddr tGw = inet_addr(szGw);

		// set the default gateway destination which is "0.0.0.0"
		char szDest[MAX_CB_IPADDRESS] = {"***.***.***.***"};
		strcpy(szDest, "0.0.0.0");
		IPAddr tDest = inet_addr(szDest);

		DWORD dwMetric = 1;  // set the default gateway metric

		MIB_IPFORWARDROW tRouteEntry;  // ip routing table row entry
		memset(&tRouteEntry, 0, sizeof(MIB_IPFORWARDROW));  // zero tRouteEntry structure
		
		// check for invalid arguments
		if (INADDR_NONE == tInet)
		{
			wprintf(_T("ChgIfGwAddr bad interface %s\n"), szInet);
			return S_FALSE;
		}

		if ((INADDR_NONE == tRouteEntry.dwForwardMask) && 
			(0 != strcmp("255.255.255.255", szNetmask)))
		{
			wprintf(_T("ChgIfGwAddr bad netmask %s\n"), szNetmask);
			return S_FALSE;
		}
		else
			tRouteEntry.dwForwardMask = tNetmask;

		if (INADDR_NONE == tRouteEntry.dwForwardNextHop )
		{
			wprintf(_T("ChgIfGwAddr bad gateway %s\n"), szGw);
			return S_FALSE;
		}
		else
			tRouteEntry.dwForwardNextHop = tGw;

		if (INADDR_NONE == tRouteEntry.dwForwardDest)
		{
			wprintf(_T("ChgIfGwAddr bad destination %s\n"), szDest);
			return S_FALSE;
		}
		else
			tRouteEntry.dwForwardDest = tDest;

		if ((tRouteEntry.dwForwardDest & tRouteEntry.dwForwardMask) != 
			 tRouteEntry.dwForwardDest)
		{
			wprintf(_T("ChgIfGwAddr invalid netmask %s\n"), szNetmask);
			return S_FALSE;
		}

		// get interface index number
		DWORD dwIfIndex = 0;
		HRESULT hr = GetIfIndex(psuIf, &dwIfIndex);
		if (S_OK != hr) throw (hr);

		// check if we have an interface with this tInet address
		DWORD dwIpAddrContext = 0;
		hr = GetIpAddrContext(dwIfIndex, psuInet, psuNetmask, &dwIpAddrContext);
		if ((S_OK != hr) && !dwIpAddrContext) throw (hr);
	
		// initialize sockets library
		WORD wVersionRequested = MAKEWORD(1,1);
		WSADATA wsaData;
		nr = WSAStartup(wVersionRequested, &wsaData);
		if (wsaData.wVersion != wVersionRequested) throw (nr); // wrong windows sockets version

		// setup and initialize varibles used within if statement blocks
		BOOL bEntryFound = FALSE;
		DWORD dwr = 0;
		PMIB_IPFORWARDTABLE ptIpRouteTab = NULL; // ip routing table
		TCHAR suIfName[MAX_ADAPTER_NAME_LENGTH] = {_T("\0")},
			  suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};

		if (bAdd)  // add or change default gateway setting
		{
			tRouteEntry.dwForwardIfIndex = dwIfIndex;
			tRouteEntry.dwForwardMetric1 = dwMetric;

			// some default values
			tRouteEntry.dwForwardProto = MIB_IPPROTO_LOCAL;
			tRouteEntry.dwForwardMetric2 = (DWORD)-1;
			tRouteEntry.dwForwardMetric3 = (DWORD)-1;
			tRouteEntry.dwForwardMetric4 = (DWORD)-1;
    
			dwr = SetIpForwardEntry(&tRouteEntry); 
			if (NO_ERROR != dwr) throw (dwr);

			if (bPersist)  // persist add default gateway changes
			{
				_tcsset(suIfIpRegValue, '\0');
				hr = GetIfName(dwIfIndex, suIfName);
				if (S_OK != hr) throw (hr);
				hr = GetIfIpRegValue(suIfName, _T("DefaultGateway"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				if (!_tcscmp(_T("0.0.0.0"), suIfIpRegValue))
					_tcscpy(suIfIpRegValue, psuGw);
				else
					hr = ChgIfIpMultiValue(suIfIpRegValue, psuGw, TRUE, FALSE, &bEntryFound);
				if (!bEntryFound)
				{
					hr = SetIfIpRegValue(suIfName, _T("DefaultGateway"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
				}
			}
		}
		
		else  // delete default gateway setting
		{
			// query for buffer size needed
			DWORD dwActualSize = 0;
			DWORD dwr = GetIpForwardTable(ptIpRouteTab, &dwActualSize, TRUE);
			if (ERROR_INSUFFICIENT_BUFFER == dwr)  // need more space
			{
				ptIpRouteTab = (PMIB_IPFORWARDTABLE)malloc(dwActualSize);
				dwr = GetIpForwardTable(ptIpRouteTab, &dwActualSize, TRUE);
			}

			if (NO_ERROR != dwr) throw (dwr);
			
			for (INT nIndex = 0; nIndex < (int)ptIpRouteTab->dwNumEntries; nIndex++)
			{
				if ((ptIpRouteTab->table[nIndex].dwForwardDest == tDest) &&
					(ptIpRouteTab->table[nIndex].dwForwardNextHop == tGw))
				{
					memcpy(&tRouteEntry, &(ptIpRouteTab->table[nIndex]), sizeof(MIB_IPFORWARDROW));
					dwr = DeleteIpForwardEntry(&tRouteEntry); 
					break;  // jump out of for loop since entry we care about
							// at this point should have now been deleted
				}
			}

			if (ptIpRouteTab)
				free(ptIpRouteTab);

			if (NO_ERROR != dwr) throw (dwr);

			if (bPersist)  // persist delete default gateway changes
			{
				_tcsset(suIfIpRegValue, '\0');
				hr = GetIfName(dwIfIndex, suIfName);
				if (S_OK != hr) throw (hr);
				hr = GetIfIpRegValue(suIfName, _T("DefaultGateway"), REG_MULTI_SZ, suIfIpRegValue);
				if (S_OK != hr) throw (hr);
				hr = ChgIfIpMultiValue(suIfIpRegValue, psuGw, FALSE, FALSE, &bEntryFound);
				if (bEntryFound)
				{
					hr = SetIfIpRegValue(suIfName, _T("DefaultGateway"), REG_MULTI_SZ, suIfIpRegValue);
					if (S_OK != hr) throw (hr);
				}
			}
		}
		nr = WSACleanup();
		if (WSAENETDOWN == nr) throw (nr);

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		INT nr = WSACleanup();
		if (WSAENETDOWN == nr) throw;
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIfIndex(LPCTSTR psuIf, DWORD *pdwIfIndex)
{
	try
	{	
		// ensure interface argument passed had the correct syntax
		INT nr = _tcsncmp(_T("if"), psuIf, sizeof(_T("if")));
		if (!nr) throw (GET_LAST_ERROR);

		nr = sizeof(_T("if")) - sizeof(TCHAR) * 2;
		// note - sizeof(TCHAR) * 2 = - 1 for null and - 1 to account for zero based array
		// use GetFriendlyIfIndex(ptIfsInfo->Index) value which is 
		// non-zero based but always unique
		DWORD dwIfFriendlyIndex = _tcstoul(&psuIf[nr], NULL, 16);
		// OR use ptIfsInfo->ComboIndex value which is zero based but
		// doesn't appear to be always unique for each adapter in all cases
		// DWORD dwIfComboIndex = _tcstoul(&szIf[nr], NULL, 16);

		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((ERROR_SUCCESS != dwr) &&
			(ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = NULL;
		ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);
	
		// get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (dwr) throw (dwr);

		while (ptIfsInfo)
		{
			// use GetFriendlyIfIndex(ptIfsInfo->Index) value which is
			// non-zero based but always unique
			if (dwIfFriendlyIndex == GetFriendlyIfIndex(ptIfsInfo->Index))
			// OR use ptIfsInfo->ComboIndex value which is zero based but
			// doesn't appear to be always unique for each adapter in all cases
			// if (dwIfComboIndex == ptIfsInfo->ComboIndex)
			{
				*pdwIfIndex = ptIfsInfo->Index;
				// OR
				// *pdwIfIndex = ptIfsInfo->ComboIndex);
				break;   // found interface index so break from loop
			}
			ptIfsInfo = ptIfsInfo->Next;
		}

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIfName(DWORD dwIfIndex, LPTSTR psuIfName)
{
	try
	{
		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((ERROR_SUCCESS != dwr) &&
			(ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = NULL;
		ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);
	
		// get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (dwr) throw (dwr);

		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info so break from loop
			}
			ptIfsInfo = ptIfsInfo->Next;
		}

		if (!ptIfInfo)
			return S_FALSE;
		
		INT nr = mbstowcs(psuIfName, ptIfInfo->AdapterName, MAX_ADAPTER_NAME_LENGTH);
		if (!nr) throw (GET_LAST_ERROR);

		return S_OK;
	}
	
	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIfDhcpMode(DWORD dwIfIndex, DWORD *pdwDhcpEnabled)
{
	try
	{		
		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((ERROR_SUCCESS != dwr) &&
			(ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = NULL;
		ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);
	
		// get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (dwr) throw (dwr);

		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info so break from loop
			}
			ptIfsInfo = ptIfsInfo->Next;
		}

		if (!ptIfInfo)
			return S_FALSE;


		*pdwDhcpEnabled = ptIfInfo->DhcpEnabled;	
    
		// if we wanted to we could optionally return dhcpEnabled value
		// based on currently persisted IfIpRegValue "EnableDHCP"
		/*
		TCHAR suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
		hr = GetIfName(dwIfIndex, suIfName);
		TCHAR suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
		hr = GetIfIpRegValue(suIfName, _T("EnableDHCP"), REG_DWORD, suIfIpRegValue);
		*pdwDhcpEnabled = _tcstoul(suIfIpRegValue, NULL, 16);
		*/
		
		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIfDhcpMac(DWORD dwIfIndex, DWORD *pdwDhcpMac)
{
	try
	{		
		// get sizing information about all adapters
		DWORD dwIfsInfoSize = 0;
		DWORD dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((ERROR_SUCCESS != dwr) &&
			(ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = NULL;
		ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);
	
		// get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (dwr) throw (dwr);

		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info so break from loop
			}
			ptIfsInfo = ptIfsInfo->Next;
		}

		if (!ptIfInfo)
			return S_FALSE;

		// *pdwDhcpEnabled = ptIfInfo->DhcpMac;	
		// no in memory parameter exists for the DhcpMac address
		// therefore we will dig it out of the registry
		TCHAR suIfName[MAX_ADAPTER_NAME_LENGTH] = {_T("\0")};
		HRESULT hr = GetIfName(dwIfIndex, suIfName);
		TCHAR suIfIpRegValue[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
		hr = GetIfIpRegValue(suIfName, _T("DhcpClientIdentifier"), REG_DWORD, suIfIpRegValue);
		DWORD dwDhcpMacInReg = _tcstoul(suIfIpRegValue, NULL, 16);
		*pdwDhcpMac = ntohl(dwDhcpMacInReg);  // return value in host order

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIpAddrContext(DWORD dwIfIndex, LPCTSTR psuInet, LPCTSTR psuNetmask, DWORD *pdwIpAddrContext)
{
	try
	{
		HRESULT hr = E_FAIL;
		DWORD dwr = 0, dwIfsInfoSize = 0;
		int nr = 0;
		
		PIP_ADDR_STRING pAddrStr = NULL;
		TCHAR suInetTmp[MAX_CB_IPADDRESS] = {_T("\0")},
			  suNetmaskTmp[MAX_CB_IPADDRESS] = {_T("\0")};

		// get sizing information about all adapters
		dwr = GetAdaptersInfo(NULL, &dwIfsInfoSize);
		if ((ERROR_SUCCESS != dwr) &&
			(ERROR_BUFFER_OVERFLOW != dwr)) throw (dwr);

		// allocate memory from sizing information
		PIP_ADAPTER_INFO ptIfsInfo = NULL;
		ptIfsInfo = (PIP_ADAPTER_INFO)GlobalAlloc(GPTR, dwIfsInfoSize);
		if (!ptIfsInfo) throw (GET_LAST_ERROR);

		// get actual adapter information
		dwr = GetAdaptersInfo(ptIfsInfo, &dwIfsInfoSize);
		if (dwr) throw (dwr);

		PIP_ADAPTER_INFO ptIfInfo = NULL;
		while (ptIfsInfo)
		{
			if (dwIfIndex == ptIfsInfo->Index)
			{
				ptIfInfo = ptIfsInfo;
				break;  // found interface info so break from loop
			}
			ptIfsInfo = ptIfsInfo->Next;
		}

		*pdwIpAddrContext = 0;  // initialize return value
		if (ptIfInfo && !ptIfInfo->DhcpEnabled)
		{
			pAddrStr = &(ptIfInfo->IpAddressList);
			while(pAddrStr)
			{
				_tcsnset(suInetTmp, '\0', sizeof(TCHAR) * MAX_CB_IPADDRESS);
				nr = mbstowcs(suInetTmp, pAddrStr->IpAddress.String, MAX_CB_IPADDRESS);

				_tcsnset(suNetmaskTmp, '\0', sizeof(TCHAR) * MAX_CB_IPADDRESS);
				nr = mbstowcs(suNetmaskTmp, pAddrStr->IpMask.String, MAX_CB_IPADDRESS);
				
				if (!_tcscmp(suInetTmp, psuInet) &&
					!_tcscmp(suNetmaskTmp, psuNetmask))
				{
					*pdwIpAddrContext = pAddrStr->Context;
					break;  // found ip address context so break from loop
				}
				else
					pAddrStr = pAddrStr->Next;
			}
		}

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}


HRESULT CAdapters::GetIfIpRegValue(LPCTSTR psuIfName, LPCTSTR psuValueName, DWORD dwValueType, LPTSTR psuValue)  // todo - switch to , VARIANT *pvValue)
{
	// note - all IfSubKey variables refer to the hive path for interface ip settings
	// used on all w2k hosts and newer.  All LegacyIfSubKey variables refer to the
	// hive path that was used on pre-w2k hosts and is provided on w2k hosts to
	// enable compatibility with legacy applications that do readonly queries against
	// that hive for interface ip values.
	// todo - update this routine to use GetVersionEx() and subsequently do the 
	// behave compatibly when running on w2k and pre-w2k hosts, i.e. windows nt 4.0 w/ sp4.

	// setup and initialize variables that need to be accessed in catch routine
	HKEY hKey = HKEY_LOCAL_MACHINE, hIfSubKey = 0, hLegacyIfSubKey = 0;
	try
	{
		// TCHAR suIfSubKey[MAX_PATH] = {_T("System\\CurrentControlSet\\Services\\")};
		// _tcscat(suIfSubKey, _T("Tcpip\\Parameters\\Interfaces\\"));
		// _tcscat(suIfSubKey, psuIfName);
		TCHAR suLegacyIfSubKey[MAX_PATH] = {_T("System\\CurrentControlSet\\Services\\")};	
		_tcscat(suLegacyIfSubKey, psuIfName);
		_tcscat(suLegacyIfSubKey, _T("\\Parameters\\Tcpip"));
		
		// HRESULT hr = OpenRegKey(NULL, hKey, suIfSubKey, KEY_QUERY_VALUE, &hIfSubKey);
		// if (S_OK != hr) throw (hr);
		HRESULT hr = OpenRegKey(NULL, hKey, suLegacyIfSubKey, KEY_QUERY_VALUE, &hLegacyIfSubKey);
		if (S_OK != hr) throw (hr);

		// setup and initialize variables used within switch block
		DWORD cbValue = 0, dwValue = 0;
		INT nIndex = 0;
		LONG lr = 0;
		_tcsset(psuValue, '\0');  // ensure our return buffer is empty first
		switch (dwValueType)
		{
		case REG_MULTI_SZ:
			cbValue = sizeof(TCHAR) * MAX_CB_REG_MULTI_SZ;
			// we need to feed our reg_multi_sz value into a byte array
			// so as to enable us to convert null separators to spaces
			// to allow for easier string manipulation of reg_multi_sz
			unsigned char aValue[sizeof(TCHAR) * MAX_CB_REG_MULTI_SZ];  // = {"\0"}
			strset((LPSTR)aValue, '\0');
			// lr = RegQueryValueEx(hIfSubKey, psuValueName, 0, &dwValueType, &aValue[0], &cbValue);
			// if (lr) throw (GET_LAST_ERROR);
			lr = RegQueryValueEx(hLegacyIfSubKey, psuValueName, 0, &dwValueType, &aValue[0], &cbValue);
			if (lr) throw (GET_LAST_ERROR);

			if (cbValue)  // check if reg_multi_sz NULL entry was obtained
			{
				// adjust count of bytes for the extra null that terminate a reg_multi_sz
				cbValue -= (sizeof(TCHAR) * 2); 
				for (nIndex = 0; nIndex < (int)cbValue; nIndex += sizeof(TCHAR))
				{
					if (aValue[nIndex])
						_tcsncat(psuValue, (LPCTSTR)&aValue[nIndex], 1);
					else
						_tcscat(psuValue, _T(" "));
				}
			}
			break;

		case REG_SZ:
			cbValue = sizeof(TCHAR) * MAX_CB_REG_SZ;
			// lr = RegQueryValueEx(hIfSubKey, psuValueName, 0, &dwValueType, (unsigned char *)&psuValue[0], &cbValue);
			// if (lr) throw (GET_LAST_ERROR);
			lr = RegQueryValueEx(hLegacyIfSubKey, psuValueName, 0, &dwValueType, (unsigned char *)&psuValue[0], &cbValue);
			if (lr) throw (GET_LAST_ERROR);
			break;
		
		case REG_DWORD:
			cbValue = sizeof(DWORD);
			// lr = RegQueryValueEx(hIfSubKey, psuValueName, 0, &dwValueType, (unsigned char *)&psuValue[0], &cbValue);
			// if (lr) throw (GET_LAST_ERROR);
			lr = RegQueryValueEx(hIfSubKey, psuValueName, 0, &dwValueType, (unsigned char *)&psuValue[0], &cbValue);
			if (lr)
			{
				INT ir = _tcscmp(_T("DhcpClientIdentifier"), psuValueName);
				// if ValueName being looked up is something other than
				// DhcpClientIdentifier where it is ok for no value to have
				// been found i.e. ERROR_INVALID_HANDLE = 6L then throw error
				if (ir) throw (GET_LAST_ERROR);
			}
			// note - don't throw error if we were looking for a DhcpClientIdentifier key
			// and it didn't return a value since it errored just because the key
			// was not present therefore we don't care about the error
			break;

		default:
			break;
		}

		// hr = CloseRegKey(NULL, hKey, hIfSubKey);
		// if (S_OK != hr) throw (hr);
		hr = CloseRegKey(NULL, hKey, hLegacyIfSubKey);
		if (S_OK != hr) throw (hr);

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		// hr = CloseRegKey(NULL, hKey, hIfSubKey);
		// if (S_OK != hr) throw;
		hr = CloseRegKey(NULL, hKey, hLegacyIfSubKey);
		if (S_OK != hr) throw;
		return S_FALSE;
	}
}


HRESULT CAdapters::ChgIfIpMultiValue(LPTSTR psuValue, LPCTSTR psuEntry, BOOL bAdd, BOOL bDupesAllowed, BOOL *pbEntryFound)
{	
	LPCTSTR psuSubStrStart = NULL;
	psuSubStrStart = _tcsstr(psuValue, psuEntry);

	*pbEntryFound = FALSE;
	if (psuSubStrStart)
		*pbEntryFound = TRUE;

	if (bAdd && (!psuSubStrStart || bDupesAllowed))  // add the reg_multi_sz entry
	{
		if (_tcslen(psuValue)) 
		{
			// check if starting reg_multi_sz value passed is NULL
			// which decides whether or not we need to add an entry
			// separator
			_tcscat(psuValue, _T(" "));
		}

		_tcscat(psuValue, psuEntry);
	}

	if (!bAdd && psuSubStrStart)  // delete the reg_multi_sz entry
	{
		// todo - build a while (pSubStrStart) loop so that
		// we can pick off the last occurrance of the address
		// substring which in the case of the subnetmask string
		// is likely the one we should remove to be safe in 
		// cases where an interface has a mix of duplicate
		// and differing vlan inet/netmasks combinatins assigned
		// OR we may want to do something different in cases
		// where bDupesAllowed = TRUE vs FALSE for deletes.

		LPCTSTR psuSubStrEnd = NULL;
		psuSubStrEnd = psuSubStrStart + _tcslen(psuEntry);

		TCHAR suBuffer[MAX_CB_REG_MULTI_SZ] = {_T("\0")};
		if (_tcslen(psuSubStrEnd))
		{
			if (psuSubStrStart == psuValue)  // reg_multi_sz entry being replaced is first in list
				_tcscpy(suBuffer, psuSubStrEnd + 1);
			else // reg_multi_sz entry being replaced is embedded in the list
			{
				_tcsncpy(suBuffer, psuValue, psuSubStrStart - psuValue - 1);	
				_tcsncat(suBuffer, psuSubStrEnd, _tcslen(psuSubStrEnd));
			}
		}
		else
		{
			if (psuSubStrStart == psuValue) // reg_multi_sz entry being replaced is only entry in list
				_tcsset(suBuffer, '\0');  // therefore zero reg_multi_sz list
			else  // reg_multi_sz entry being replaced is at end of in list
				_tcsncpy(suBuffer, psuValue, psuSubStrStart - psuValue - 1);		
		}
		_tcscpy(psuValue, suBuffer);
	}	
	
	return S_OK;
}


HRESULT CAdapters::SetIfIpRegValue(LPCTSTR psuIfName, LPCTSTR psuValueName, DWORD dwValueType, LPCTSTR psuValue)  // todo - switch to , VARIANT *pvValue)
{
	// note - all IfSubKey variables refer to the hive path for interface ip settings
	// used on all w2k hosts and newer.  All LegacyIfSubKey variables refer to the
	// hive path that was used on pre-w2k hosts and is provided on w2k hosts to
	// enable compatibility with legacy applications that do readonly queries against
	// that hive for interface ip values.
	// todo - update this routine to use GetVersionEx() and subsequently do the 
	// behave compatibly when running on w2k and pre-w2k hosts, i.e. windows nt 4.0 w/ sp4.

	// setup and initialize variables that need to be accessed in catch routine
	HKEY hKey = HKEY_LOCAL_MACHINE, hIfSubKey = 0, hLegacyIfSubKey = 0;
	try
	{
		TCHAR suIfSubKey[MAX_PATH] = {_T("System\\CurrentControlSet\\Services\\")};
		_tcscat(suIfSubKey, _T("Tcpip\\Parameters\\Interfaces\\"));
		_tcscat(suIfSubKey, psuIfName);
		TCHAR suLegacyIfSubKey[MAX_PATH] = {_T("System\\CurrentControlSet\\Services\\")};
		_tcscat(suLegacyIfSubKey, psuIfName);
		_tcscat(suLegacyIfSubKey, _T("\\Parameters\\Tcpip"));
		
		HRESULT hr = OpenRegKey(NULL, hKey, suIfSubKey, KEY_SET_VALUE, &hIfSubKey);
		if (S_OK != hr) throw (hr);
		hr = OpenRegKey(NULL, hKey, suLegacyIfSubKey, KEY_SET_VALUE, &hLegacyIfSubKey);
		if (S_OK != hr) throw (hr);

		// setup and initialize variables used within switch block
		DWORD cbValue = 0, dwValue = 0;
		INT nIndex = 0;
		LONG lr = 0;
		switch (dwValueType)
		{
		case REG_MULTI_SZ:
			// we need to feed our reg_multi_sz value into a byte array
			// so as to enable us to convert spaces to null separators
			// the reg_multi_sz value was created in this way to allow for
			// easier string manipulation of reg_multi_sz before writing
			unsigned char aValue[sizeof(TCHAR) * MAX_CB_REG_MULTI_SZ];  // = {"\0"}
			strset((LPSTR)aValue, '\0');
			cbValue = _tcslen(psuValue);
			if (cbValue)  // check if reg_multi_sz NULL entry was passed
			{
				for (nIndex = 0; nIndex < (int)cbValue; nIndex++)
				{
					if (_T(' ') != psuValue[nIndex])
					{
						strncpy((LPSTR)&aValue[sizeof(TCHAR) * nIndex], (LPCSTR)&psuValue[nIndex], 1);
						aValue[sizeof(TCHAR) * nIndex + 1] = '\0';
					}
					else
					{
						// separate reg_multi_sz entries will a null
						aValue[sizeof(TCHAR) * nIndex] = '\0'; 
					}
				}

				// adjust count of bytes to account for bytes used for unicode string representation
				cbValue *= sizeof(TCHAR);

				// add two unicode null's which terminates the reg_multi_sz
				aValue[sizeof(TCHAR) * nIndex] = '\0';
				aValue[sizeof(TCHAR) * nIndex + 1] = '\0';
				aValue[sizeof(TCHAR) * nIndex + 2] = '\0';
				aValue[sizeof(TCHAR) * nIndex + 3] = '\0';

				// adjust count of bytes to account for extra unicode null terminating reg_multi_sz
				cbValue += sizeof(TCHAR) * 2;
			}
			else   // reg_multi_sz NULL entry was passed
			{
				strset((char *)aValue, '\0');

				// adjust count of bytes to account for extra unicode null terminating reg_multi_sz
				cbValue += sizeof(TCHAR);
			}

			lr = RegSetValueEx(hIfSubKey, psuValueName, 0, dwValueType, &aValue[0], cbValue);
			if (lr) throw (lr);
			lr = RegSetValueEx(hLegacyIfSubKey, psuValueName, 0, dwValueType, &aValue[0], cbValue);
			if (lr) throw (lr);
			break;

		case REG_SZ:
			cbValue = sizeof(TCHAR) * _tcslen(psuValue);
			lr = RegSetValueEx(hIfSubKey, psuValueName, 0, dwValueType, (const unsigned char *)&psuValue[0], cbValue);
			if (lr) throw (lr); 
			lr = RegSetValueEx(hLegacyIfSubKey, psuValueName, 0, dwValueType, (const unsigned char *)&psuValue[0], cbValue);
			if (lr) throw (lr); 
			break;

		case REG_DWORD:
			cbValue = sizeof(DWORD);
			dwValue = _tcstoul(psuValue, NULL, 16);
			lr = RegSetValueEx(hIfSubKey, psuValueName, 0, dwValueType, (const unsigned char *)&dwValue, cbValue);
			if (lr) throw (lr);
			lr = RegSetValueEx(hLegacyIfSubKey, psuValueName, 0, dwValueType, (const unsigned char *)&dwValue, cbValue);
			if (lr) throw (lr);
			break;

		default:
			break;
		}

		hr = CloseRegKey(NULL, hKey, hIfSubKey);
		if (S_OK != hr) throw (hr);
		hr = CloseRegKey(NULL, hKey, hLegacyIfSubKey);
		if (S_OK != hr) throw (hr);

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		hr = CloseRegKey(NULL, hKey, hIfSubKey);
		if (S_OK != hr) throw;
		hr = CloseRegKey(NULL, hKey, hLegacyIfSubKey);
		if (S_OK != hr) throw;
		return S_FALSE;
	}
}


HRESULT CAdapters::OpenRegKey(LPCTSTR psuHostName, HKEY hKey, LPCTSTR psuSubKey, DWORD dwSamDesired, PHKEY phSubKey)
{	
	try
	{
		if (psuHostName)
		{
			TCHAR suUncPath[MAX_PATH] = {_T("\\\\")};
			_tcscat(suUncPath, psuHostName);
			LONG lr = RegConnectRegistry(suUncPath, hKey, &hKey);
			if (ERROR_SUCCESS != lr) throw (lr);
		}
			
		// commonly used dwSamDesired values are KEY_SET_VALUE | KEY_QUERY_VALUE
		long lr = RegOpenKeyEx(hKey, psuSubKey, 0, dwSamDesired, phSubKey);
		if (ERROR_SUCCESS != lr) throw (lr);
		
		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}

}


HRESULT CAdapters::CloseRegKey(LPCTSTR psuHostName, HKEY hKey, HKEY hSubKey)
{	
	try
	{
		LONG lr = RegCloseKey(hSubKey);
		if (ERROR_SUCCESS != lr) throw (lr);

		lr = RegCloseKey(hKey);
		if (ERROR_SUCCESS != lr) throw (lr);

		if (psuHostName)
		{
			TCHAR suUncPath[MAX_PATH] = {_T("\\\\")};
			_tcscat(suUncPath, psuHostName);
			_tcscat(suUncPath, _T("\\ipc$"));

			DWORD dwr = WNetCancelConnection2(suUncPath, 0, FALSE);
			if (NO_ERROR != dwr) throw (dwr);
		}

		return S_OK;
	}

	catch (HRESULT hrThrown)
	{
		HRESULT hr = DisplayFormatMessage(hrThrown);
		return S_FALSE;
	}
}
