// ArgV.cpp: implementation of the CArgV class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ifConfig.h"
#include "ArgV.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CArgV::CArgV()
{
	m_dwArgs = 0; // initialize arguments bit mask
	
	// alloc memory for member variable strings
    m_suIfArg = SysAllocString(_T("if00"));
	m_suDhcpMac = SysAllocString(_T("00000000"));
	m_suInet = SysAllocString(_T("000.000.000.000"));
	m_suNetmask = SysAllocString(_T("000.000.000.000"));
	m_suGw = SysAllocString(_T("000.000.000.000"));

}

CArgV::~CArgV()
{
	// free memory for member variable strings
	SysFreeString(m_suIfArg);
	SysFreeString(m_suDhcpMac);
	SysFreeString(m_suInet);
	SysFreeString(m_suNetmask);
	SysFreeString(m_suGw);
}

HRESULT CArgV::ExtractArgs(int argc, TCHAR *argv[])
{
	if (1 == argc)
		m_dwArgs |= ARG_HELP; // turn on help
	else
	{
		#ifdef _DEBUG
			m_dwArgs |= ARG_HELP; // test with help flag set
		#endif

		// ensure help flag is current turned off
		m_dwArgs = (m_dwArgs & ARG_HELP) ? (m_dwArgs ^= ARG_HELP) : (m_dwArgs);
		
		// setup and initialize variables used within loop
		BOOL bFoundArg = FALSE;
		HRESULT hr = E_FAIL;
		CAdapters oAdapters;
		for (INT nIndex = 1; nIndex < argc; nIndex++)
		{
			bFoundArg = FALSE;

			// decide what interface(s) will be worked on
			if (!bFoundArg &&
				!_tcsncmp(_T("if"), argv[nIndex], _tcslen(_T("if"))))
			{
				_tcscpy(m_suIfArg, argv[nIndex]);
				m_dwArgs |= ARG_IF;
				bFoundArg = TRUE;
			}
			
			if (!bFoundArg &&
				(!_tcscmp(_T("-a"), argv[nIndex])))
			{
				m_dwArgs |= ARG_IF_ALL;
				bFoundArg = TRUE;
			}
			
			if (!bFoundArg &&
				(!_tcscmp(_T("-au"), argv[nIndex])))
			{
				m_dwArgs |= ARG_IF_ALL_UP;
				bFoundArg = TRUE;
			}			
			
			if (!bFoundArg &&
				(!_tcscmp(_T("-ad"), argv[nIndex])))
			{
				m_dwArgs |= ARG_IF_ALL_DOWN;
				bFoundArg = TRUE;
			}

			// decide which if any settings were provided
			if (!bFoundArg &&
				(!_tcscmp(_T("up"), argv[nIndex])))
			{
				m_dwArgs |= ARG_UP;
				bFoundArg = TRUE;
			}

			if (!bFoundArg &&
				(!_tcscmp(_T("down"), argv[nIndex])))
			{
				m_dwArgs |= ARG_DOWN;
				bFoundArg = TRUE;
			}

			if (!bFoundArg &&
				(!_tcscmp(_T("release"), argv[nIndex])))
			{
				m_dwArgs |= ARG_RELEASE;
				bFoundArg = TRUE;
			}

			if (!bFoundArg && 
				(!_tcscmp(_T("renew"), argv[nIndex])))
			{
				m_dwArgs |= ARG_RENEW;
				bFoundArg = TRUE;
			}
	
			if (!bFoundArg && 
				(!_tcscmp(_T("dhcp"), argv[nIndex])))
			{
				m_dwArgs |= ARG_DHCP;
				bFoundArg = TRUE;
			}
		
			if (!bFoundArg && 
				(!_tcscmp(_T("dhcpMac"), argv[nIndex])))
			{
				if (MAX_ADAPTER_ADDRESS_LENGTH == _tcslen(argv[nIndex + 1]))
					// 00000000 used to turn off setting
				{
					m_dwDhcpMac = 0;
					m_dwDhcpMac = _tcstoul(argv[nIndex + 1], NULL, 16);
					_ultow(m_dwDhcpMac, m_suDhcpMac, 16);
					m_dwArgs |= ARG_DHCPMAC;
					nIndex++;
					bFoundArg = TRUE;
				}
				else  // erroneous argument provided
				{
					nIndex++;
					m_dwArgs |= ARG_HELP; // turn on help
					bFoundArg = TRUE;
				}
			}

			if (!bFoundArg && 
				(_tcschr(argv[nIndex], '.')) &&
				!(m_dwArgs & ARG_INET))
			{
				_tcsset(m_suInet, '\0');
				_tcscpy(m_suInet, argv[nIndex]);
				m_dwInet = 0;
				hr = oAdapters.IpStringToHex(m_suInet, &m_dwInet);
				m_dwArgs |= ARG_INET;
				bFoundArg = TRUE;
			}

			if (!bFoundArg && 
				(MAX_ADAPTER_ADDRESS_LENGTH == _tcslen(argv[nIndex])) &&
				!(m_dwArgs & ARG_INET))
			{
				m_dwInet = 0;
				m_dwInet = _tcstoul(argv[nIndex], NULL, 16);
				_tcsset(m_suInet, '\0');
				hr = oAdapters.IpHexToString(m_dwInet, m_suInet);
				m_dwArgs |= ARG_INET;
				bFoundArg = TRUE;
			}
	
			if (!bFoundArg && 
				(!_tcscmp(_T("netmask"), argv[nIndex])))
			{
				if (_tcschr(argv[nIndex + 1], '.'))
				{
					_tcsset(m_suNetmask, '\0');
					_tcscpy(m_suNetmask, argv[nIndex + 1]);
					m_dwNetmask = 0;
					hr = oAdapters.IpStringToHex(m_suNetmask, &m_dwNetmask);
					m_dwArgs |= ARG_NETMASK;
					nIndex++;
					bFoundArg = TRUE;
				}
				if (!bFoundArg && MAX_ADAPTER_ADDRESS_LENGTH == _tcslen(argv[nIndex + 1]))
				{
					m_dwNetmask = 0;
					m_dwNetmask = _tcstoul(argv[nIndex + 1], NULL, 16);
					_tcsset(m_suNetmask, '\0');
					hr = oAdapters.IpHexToString(m_dwNetmask, m_suNetmask);
					m_dwArgs |= ARG_NETMASK;
					nIndex++;
					bFoundArg = TRUE;
				}
				if (!bFoundArg)  // erroneous argument provided
				{
					nIndex++;
					m_dwArgs |= ARG_HELP; // turn on help
					bFoundArg = TRUE;
				}

			}
			
			if (!bFoundArg && 
				(!_tcscmp(_T("gw"), argv[nIndex])))
			{
				if (_tcschr(argv[nIndex + 1], '.'))
				{
					_tcsset(m_suGw, '\0');
					_tcscpy(m_suGw, argv[nIndex + 1]);
					m_dwGw = 0;
					hr = oAdapters.IpStringToHex(m_suGw, &m_dwGw);
					m_dwArgs |= ARG_GW;
					nIndex++;
					bFoundArg = TRUE;
				}
				if (!bFoundArg && MAX_ADAPTER_ADDRESS_LENGTH == _tcslen(argv[nIndex + 1]))
				{
					m_dwGw = 0;
					m_dwGw = _tcstoul(argv[nIndex + 1], NULL, 16);
					_tcsset(m_suGw, '\0');
					hr = oAdapters.IpHexToString(m_dwGw, m_suGw);
					m_dwArgs |= ARG_GW;
					nIndex++;
					bFoundArg = TRUE;
				}
				if (!bFoundArg)  // erroneous argument provided
				{
					nIndex++;
					m_dwArgs |= ARG_HELP; // turn on help
					bFoundArg = TRUE;
				}

			}

			if (!bFoundArg && 
				(!_tcscmp(_T("add"), argv[nIndex])))
			{
				m_dwArgs |= ARG_ADD;
				bFoundArg = TRUE;
			}
			
			if (!bFoundArg && 
				(!_tcscmp(_T("del"), argv[nIndex])))
			{
				m_dwArgs |= ARG_DEL;
				bFoundArg = TRUE;
			}
			
			if (!bFoundArg && 
				(!_tcscmp(_T("persist"), argv[nIndex])))
			{
				m_dwArgs |= ARG_PERSIST;
				bFoundArg = TRUE;
			}

			// set help flag if user provided incorrect switch, argument
			// or attempted to request help using common help flags
			// which would end up here as well with bFoundArg = FALSE
			if (!bFoundArg)
				m_dwArgs |= ARG_HELP; // turn on help
		}
	}
	
	return S_OK;
}

HRESULT CArgV::ProcessArgs()
{
	HRESULT hr = E_FAIL;
	BOOL bDone = FALSE;
	CAdapters oAdapters;

	// help
	if (m_dwArgs & ARG_HELP)
	{
		hr = Help();
		bDone = TRUE;
	}
	
	// list
	if (!bDone 
			&& ((m_dwArgs & ARG_IF) ||
				(m_dwArgs & ARG_IF_ALL) ||
				(m_dwArgs & ARG_IF_ALL_UP) ||
				(m_dwArgs & ARG_IF_ALL_DOWN))
			&& !((m_dwArgs & ARG_UP) || 
			     (m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RENEW) ||
				 (m_dwArgs & ARG_RELEASE) ||
				 (m_dwArgs & ARG_DHCP) ||
				 (m_dwArgs & ARG_DHCPMAC) ||
				 (m_dwArgs & ARG_INET) ||
				 (m_dwArgs & ARG_NETMASK) ||
				 (m_dwArgs & ARG_GW) ||
				 (m_dwArgs & ARG_ADD) ||
				 (m_dwArgs & ARG_DEL) ||
				 (m_dwArgs & ARG_PERSIST)))
	{		
		if (m_dwArgs & ARG_IF)
			hr = oAdapters.List(m_suIfArg);
		if (m_dwArgs & ARG_IF_ALL)
			hr = oAdapters.List(_T("all"));
		if (m_dwArgs & ARG_IF_ALL_UP)
			hr = oAdapters.List(_T("up"));
		if (m_dwArgs & ARG_IF_ALL_DOWN)
			hr = oAdapters.List(_T("down"));
		bDone = TRUE;
	}

	// up
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN))
			&& (m_dwArgs & ARG_UP)
			&& !((m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RENEW) ||
				 (m_dwArgs & ARG_RELEASE) ||
				 (m_dwArgs & ARG_DHCP) ||
				 (m_dwArgs & ARG_DHCPMAC) ||
				 (m_dwArgs & ARG_INET) ||
				 (m_dwArgs & ARG_NETMASK) ||
				 (m_dwArgs & ARG_GW) ||
				 (m_dwArgs & ARG_ADD) ||
				 (m_dwArgs & ARG_DEL)))
	{
		hr = oAdapters.ChgIfStatus(m_suIfArg, TRUE);
		hr = oAdapters.List(m_suIfArg);  // display updated settings
		bDone = TRUE;
	}

	// down
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN) ||
				 (m_dwArgs & ARG_UP))
			&& (m_dwArgs & ARG_DOWN)
			&& !((m_dwArgs & ARG_RENEW) ||
				(m_dwArgs & ARG_RELEASE) ||
				(m_dwArgs & ARG_DHCP) ||
				(m_dwArgs & ARG_DHCPMAC) ||
				(m_dwArgs & ARG_INET) ||
				(m_dwArgs & ARG_NETMASK) ||
				(m_dwArgs & ARG_GW) ||
				(m_dwArgs & ARG_ADD) ||
				(m_dwArgs & ARG_DEL)))
	{
		hr = oAdapters.ChgIfStatus(m_suIfArg, FALSE);
		hr = oAdapters.List(m_suIfArg);  // display updated settings
		bDone = TRUE;
	}

	// renew
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN) ||
				 (m_dwArgs & ARG_UP) ||
				 (m_dwArgs & ARG_DOWN))
			&& (m_dwArgs & ARG_RENEW)
			&& !((m_dwArgs & ARG_RELEASE) ||
				 (m_dwArgs & ARG_DHCP) ||
				 (m_dwArgs & ARG_DHCPMAC) ||
				 (m_dwArgs & ARG_INET) ||
				 (m_dwArgs & ARG_NETMASK) ||
				 (m_dwArgs & ARG_GW) ||
				 (m_dwArgs & ARG_ADD) ||
				 (m_dwArgs & ARG_DEL)))
	{
		hr = oAdapters.ChgIfDhcpStatus(m_suIfArg, TRUE);
		hr = oAdapters.List(m_suIfArg);  // display updated settings
		bDone = TRUE;
	}

	// release
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN) ||
				 (m_dwArgs & ARG_UP) ||
				 (m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RENEW))
			&& (m_dwArgs & ARG_RELEASE)
			&& !((m_dwArgs & ARG_DHCP) ||
				 (m_dwArgs & ARG_DHCPMAC) ||
				 (m_dwArgs & ARG_INET) ||
				 (m_dwArgs & ARG_NETMASK) ||
				 (m_dwArgs & ARG_GW) ||
				 (m_dwArgs & ARG_ADD) ||
				 (m_dwArgs & ARG_DEL)))
	{
		hr = oAdapters.ChgIfDhcpStatus(m_suIfArg, FALSE);
		hr = oAdapters.List(m_suIfArg);  // display updated settings
		bDone = TRUE;
	}

	// dhcp and/or dhcpMac
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN) ||
				 (m_dwArgs & ARG_UP) ||
				 (m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RENEW) ||
				 (m_dwArgs & ARG_RELEASE))
			&& ((m_dwArgs & ARG_DHCP) ||
				(m_dwArgs & ARG_DHCPMAC))
			&& !((m_dwArgs & ARG_INET) ||
				 (m_dwArgs & ARG_NETMASK) ||
				 (m_dwArgs & ARG_GW) ||
				 (m_dwArgs & ARG_ADD) ||
				 (m_dwArgs & ARG_DEL)))
	{			
		// if ARG_DHCPMAC set with dhcp switch then call
		// dhcpMac in addition to dhcp to capture
		// cases when user is switching to dhcp and
		// desires to set the dhcp mac address in the 
		// same pass OR just to ensure that dhcp is in
		// fact enabled when they set this parameter.
		if (m_dwArgs & ARG_DHCPMAC)
			hr = oAdapters.ChgIfDhcpMac(m_suIfArg, m_dwDhcpMac);  // (m_dwArgs & ARG_PERSIST)); is implied
	
		if (m_dwArgs & ARG_DHCP)
			hr = oAdapters.ChgIfDhcpAddr(m_suIfArg, TRUE, (m_dwArgs & ARG_PERSIST));

		hr = oAdapters.List(m_suIfArg);  // display updated settings
		bDone = TRUE;
	}

	// add
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_IF_ALL) ||
				 (m_dwArgs & ARG_IF_ALL_UP) ||
				 (m_dwArgs & ARG_IF_ALL_DOWN) ||
				 (m_dwArgs & ARG_UP) ||
				 (m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RENEW) ||
				 (m_dwArgs & ARG_RELEASE) ||
			 	 (m_dwArgs & ARG_DHCP)|| 
				 (m_dwArgs & ARG_DHCPMAC))
			&& (m_dwArgs & ARG_INET)
			&& (m_dwArgs & ARG_NETMASK)
			// && (m_dwArgs & ARG_GW)
			&& (m_dwArgs & ARG_ADD)
			&& !(m_dwArgs & ARG_DEL))
	{
		// if ARG_GW set with add switch then call add
		// ipaddress in addition to add default gateway 
		// to capture cases when user is adding an address
		// and desires to set the default gateway in the 
		// same pass OR just to ensure correct interface
		// ipaddress that they are expecting does exist.
		if (m_dwArgs & ARG_INET)
			hr = oAdapters.ChgIfIpAddr(m_suIfArg, m_suInet, m_suNetmask, TRUE, (m_dwArgs & ARG_PERSIST));

		if (m_dwArgs & ARG_GW)
			hr = oAdapters.ChgIfGwAddr(m_suIfArg, m_suInet, m_suNetmask, m_suGw, TRUE, (m_dwArgs & ARG_PERSIST));

		oAdapters.List(m_suIfArg);  // display updated settings
		// todo - if (m_dwArgs & ARG_GW) should we also do a
		// route print output as well similar to the example
		// in the platform sdk iproute.cpp example?
		bDone = TRUE;
	}

	// delete
	if (!bDone 
			&& (m_dwArgs & ARG_IF)
			&& !((m_dwArgs & ARG_UP) ||
				 (m_dwArgs & ARG_DOWN) ||
				 (m_dwArgs & ARG_RELEASE) ||
				 (m_dwArgs & ARG_RENEW) ||
			 	 (m_dwArgs & ARG_DHCP) ||
				 (m_dwArgs & ARG_DHCPMAC))
			&& (m_dwArgs & ARG_INET)
			&& (m_dwArgs & ARG_NETMASK)
			// && (m_dwArgs & ARG_GW)
			&& !(m_dwArgs & ARG_ADD)
			&& (m_dwArgs & ARG_DEL))
	{
		// if ARG_GW set with del switch then only 
		// delete default gateway and not the ipaddress
		if ((m_dwArgs & ARG_INET) && !(m_dwArgs & ARG_GW))  
			hr = oAdapters.ChgIfIpAddr(m_suIfArg, m_suInet, m_suNetmask, FALSE, (m_dwArgs & ARG_PERSIST));

		if (m_dwArgs & ARG_GW)
			hr = oAdapters.ChgIfGwAddr(m_suIfArg, m_suInet, m_suNetmask, m_suGw, FALSE, (m_dwArgs & ARG_PERSIST));

		hr = oAdapters.List(m_suIfArg);  // display updated settings
		// todo - if (m_dwArgs & ARG_GW) should we also do a
		// route print output as well similar to the example
		// in the platform sdk iproute.cpp example?
		bDone = TRUE;
	}

	// if we made it to this point then the correct
	// switches or switch combinations were not 
	// provided therefore display usage help
	if (!bDone)
	{
		hr = Help();
		bDone = TRUE;
	}

	return hr;
}

HRESULT CArgV::Help()
{
	// ifconfig default output that we want to model visually and
	// feature wise as much as possible
	// usage: ifconfig <interface> | -a | -au | -ad"
    //			[ <af> ] [ <address> [ <dest_addr> ] ] [ up ] [ down ]
    //			[ auto-revarp ]
    //			[ netmask <mask> ] [ broadcast <broad_addr> ]
    //			[ metric <n> ]
    //			[ mtu <n> ]
    //			[ trailers | -trailers ] [private | -private]
    //			[ arp | -arp ]
	//			[ plumb | unplumb]

	wcout << _T(" ") << endl;
	wcout << _T("usage: ifconfig <interface> | -a | -au | -ad") << endl;
    // wcout << _T("        [ up | down ]") << endl;
	wcout << _T("        [ renew | release ]") << endl;
	wcout << _T("        [ dhcp ] [dhcpMac <mac>] |") << endl;
	wcout << _T("        [ <address> netmask <mask> [ gw <address> ] ]") << endl;
	wcout << _T("        [ add | del [ persist ] ]") << endl;
	wcout << _T(" ") << endl;
	/*
	wcout << _T("note - if switching between dhcp & static addressing then a reboot") << endl;
	wcout << _T("       is required for settings to become active.  Use the system") << endl;
	wcout << _T("       provided netsh.exe utility if you need to avoid the reboot.") << endl;
	wcout << _T(" ") << endl;
	*/
	wcout << _T("examples:") << endl;
	// wcout << _T("       ifconfig -a") << endl;
	// wcout << _T("        [lists all interfaces and settings]") << endl;
	// wcout << _T(" ") << endl;
	wcout << _T("       ifconfig -au") << endl;
	wcout << _T("        [lists all up interfaces and settings]") << endl;
	wcout << _T(" ") << endl;
	// wcout << _T("       ifconfig if0 down") << endl;
	// wcout << _T("        [shuts down interface 0]") << endl;
	// wcout << _T(" ") << endl;
	wcout << _T("       ifconfig if0 release") << endl;
	wcout << _T("        [releases dhcp acquired address on interface 0]") << endl;
	wcout << _T(" ") << endl;
	wcout << _T("       ifconfig if1 dhcp dhcpMac 1234abcd persist") << endl;
	wcout << _T("        [enables dhcp on if1 for next boot and sets mac used in request]") << endl;
	wcout << _T(" ") << endl;
	wcout << _T("       ifconfig if2 169.254.0.4 netmask 255.255.0.0 add persist") << endl;
	wcout << _T("        [adds address/netmask to if2 and persists the change]") << endl;
	wcout << _T("        [if iterface was previously dhcp see notes above]") << endl;
	wcout << _T(" ") << endl;	
	// wcout << _T("       ifconfig if2 169.254.0.4 netmask 255.255.0.0 del") << endl;
	// wcout << _T("        [deletes address/netmask from if2 but does not persist the change]") << endl;
	// wcout << _T(" ") << endl;
	wcout << _T("       ifconfig if2 169.254.0.4 netmask ffff0000 gw 169.254.0.1 add") << endl;
	wcout << _T("        [adds default gateway to if2 but does not persist the change]") << endl;
	// wcout << _T(" ") << endl;	

	return S_OK;
}
